#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# gui_elements.py
# Maximilian Beeskow
# 26.06.2024
# ----------------------
#
## MODULES
# external
import tkinter as tk
from tkinter import ttk
import numpy as np
import colorsys
import matplotlib.colors
# internal
try:
    from pysills.modules.essential_functions import Essentials
except:
    from modules.essential_functions import Essentials

## CLASSES
class SimpleElements:
    #
    def __init__(self, parent, row_id, column_id, fg, bg, n_rows=1, n_columns=1):
        self.parent = parent
        self.row_id = row_id
        self.column_id = column_id
        self.n_rows = n_rows
        self.n_columns = n_columns
        self.fg = fg
        self.bg = bg
    #
    def create_simple_label(self, text, fontsize=None, relief=tk.GROOVE, textvariable=False, anchor="center",
                            sticky="nesw", link=False):
        if fontsize != None:
            if textvariable == False:
                lbl = tk.Label(self.parent, text=text, relief=relief, bg=self.bg, fg=self.fg, font=(fontsize),
                               anchor=anchor)
            else:
                lbl = tk.Label(self.parent, textvariable=text, relief=relief, bg=self.bg, fg=self.fg, font=(fontsize),
                               anchor=anchor)
        else:
            if textvariable == False:
                lbl = tk.Label(self.parent, text=text, relief=relief, bg=self.bg, fg=self.fg, font=fontsize)
            else:
                lbl = tk.Label(self.parent, textvariable=text, relief=relief, bg=self.bg, fg=self.fg, font=fontsize)

        if link == True:
            lbl.configure(cursor="hand2")

        lbl.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky=sticky)
        #
        return lbl
    #
    def create_label(self, text, relief=tk.GROOVE):
        lbl = tk.Label(self.parent, text=text, relief=relief, bg=self.bg, fg=self.fg)
        lbl.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
    #
    def create_frame(self, relief=tk.GROOVE):
        frm = tk.Frame(self.parent, bg=self.bg, bd=1, relief=relief)
        frm.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
        return frm
    #
    def create_label_variable(self, label_container, relief=tk.SUNKEN):
        var_text = tk.IntVar()
        var_text.set(0)
        lbl = tk.Label(self.parent, text=var_text.get(), relief=relief)
        label_container.append(lbl)
        lbl.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
    #
    def create_button(self, text, command=None):
        if command == None:
            btn = tk.Button(self.parent, text=text, fg=self.fg, bg=self.bg)
        else:
            btn = tk.Button(self.parent, text=text, fg=self.fg, bg=self.bg, command=command)
        btn.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
    #
    def create_simple_button(self, text, bg_active="white", fg_active="black", command=None):
        if command == None:
            btn = tk.Button(self.parent, text=text, bg=self.bg, fg=self.fg, activebackground=bg_active,
                            activeforeground=fg_active, highlightbackground=self.bg, font="sans 10 bold")
        else:
            btn = tk.Button(self.parent, text=text, bg=self.bg, fg=self.fg, activebackground=bg_active,
                            activeforeground=fg_active, highlightbackground=self.bg,
                            highlightthickness=0, command=command, font="sans 10 bold")
        btn.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
        #
        return btn
    #
    def create_time_window_entries(self, times, part, command_start=None, command_end=None, relief=tk.GROOVE):
        #
        lbl_start = tk.Label(self.parent, text="Time Start", relief=relief)
        lbl_start.grid(row=self.row_id, column=self.column_id, sticky="nesw")
        lbl_end = tk.Label(self.parent, text="Time End", relief=relief)
        lbl_end.grid(row=self.row_id+1, column=self.column_id, sticky="nesw")
        #
        if part == "BG":
            self.t_start_bg = tk.StringVar()
            self.t_start_bg.set(times.iloc[0])
            self.entr_t_start_bg = tk.Entry(self.parent, textvariable=self.t_start_bg)
            self.entr_t_start_bg.grid(row=self.row_id, column=self.column_id+1, sticky="nesw")
            if command_start == None:
                pass
            else:
                self.entr_t_start_bg.bind("<Return>", command_start)
            self.t_end_bg = tk.StringVar()
            self.t_end_bg.set(times.iloc[-1])
            self.entr_t_end_bg = tk.Entry(self.parent, textvariable=self.t_end_bg)
            self.entr_t_end_bg.grid(row=self.row_id+1, column=self.column_id+1, sticky="nesw")
            if command_end == None:
                pass
            else:
                self.entr_t_end_bg.bind("<Return>", command_end)
            #
            return self.entr_t_start_bg, self.entr_t_end_bg
        elif part == "SIG":
            self.t_start_sig = tk.StringVar()
            self.t_start_sig.set(times.iloc[0])
            self.entr_t_start_sig = tk.Entry(self.parent, textvariable=self.t_start_sig)
            self.entr_t_start_sig.grid(row=self.row_id, column=self.column_id+1, sticky="nesw")
            if command_start == None:
                pass
            else:
                self.entr_t_start_sig.bind("<Return>", command_start)
            self.t_end_sig = tk.StringVar()
            self.t_end_sig.set(times.iloc[-1])
            self.entr_t_end_sig = tk.Entry(self.parent, textvariable=self.t_end_sig)
            self.entr_t_end_sig.grid(row=self.row_id+1, column=self.column_id+1, sticky="nesw")
            if command_end == None:
                pass
            else:
                self.entr_t_end_sig.bind("<Return>", command_end)
            #
            return self.entr_t_start_sig, self.entr_t_end_sig
        elif part == "MAT":
            self.t_start_mat = tk.StringVar()
            self.t_start_mat.set(times.iloc[0])
            self.entr_t_start_mat = tk.Entry(self.parent, textvariable=self.t_start_mat)
            self.entr_t_start_mat.grid(row=self.row_id, column=self.column_id+1, sticky="nesw")
            if command_start == None:
                pass
            else:
                self.entr_t_start_mat.bind("<Return>", command_start)
            self.t_end_mat = tk.StringVar()
            self.t_end_mat.set(times.iloc[-1])
            self.entr_t_end_mat = tk.Entry(self.parent, textvariable=self.t_end_mat)
            self.entr_t_end_mat.grid(row=self.row_id+1, column=self.column_id+1, sticky="nesw")
            if command_end == None:
                pass
            else:
                self.entr_t_end_mat.bind("<Return>", command_end)
            #
            return self.entr_t_start_mat, self.entr_t_end_mat
    #
    def create_option_times(self, part, times_seg, command=None):
        option_list = []
        if len(times_seg) == 0:
            option_list.append(["No Time Intervals"])
        else:
            for i in range(len(times_seg)):
                option_list.append([str(times_seg[i][0])+" - "+str(times_seg[i][1])])
        option_list = np.array(option_list)[:, 0]
        #
        if part == "BG":
            self.var_iw_bg = tk.StringVar()
            self.var_iw_bg.set("Select Time Interval")
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_bg, *option_list)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            else:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_bg, *option_list, command=command)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
            #
            return self.var_iw_bg
        elif part == "MATR":
            self.var_iw_matr = tk.StringVar()
            self.var_iw_matr.set("Select Time Interval")
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_matr, *option_list)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            else:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_matr, *option_list, command=command)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
            #
            return self.var_iw_matr
        elif part == "INCL":
            self.var_iw_incl = tk.StringVar()
            self.var_iw_incl.set("Select Time Interval")
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_incl, *option_list)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            else:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_incl, *option_list, command=command)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
            #
            return self.var_iw_incl
        elif part == "SIG":
            self.var_iw_sig = tk.StringVar()
            self.var_iw_sig.set("Select Time Interval")
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_sig, *option_list)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            else:
                opt_menu = tk.OptionMenu(self.parent, self.var_iw_sig, *option_list, command=command)
                opt_menu.config(bg=self.bg, activebackground=self.bg)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
            #
            return self.var_iw_sig
    #
    def create_isotope_column(self, input_isotopes, relief=tk.GROOVE):
        i = 0
        try:
            for isotope in input_isotopes:
                lbl_isotope = tk.Label(self.parent, text=isotope, relief=relief)
                lbl_isotope.grid(row=self.row_id+i, column=self.column_id, sticky="nesw")
                i += 1
        except:
            for isotope in input_isotopes[:, 0]:
                lbl_isotope = tk.Label(self.parent, text=isotope, relief=relief)
                lbl_isotope.grid(row=self.row_id+i, column=self.column_id, sticky="nesw")
                i += 1
    #
    def create_signal_entries(self, var_sig_mu, entr_sig_mu, var_sig_std, entr_sig_std, input_isotopes):
        i = 0
        for isotope in input_isotopes[:, 0]:
            var_sig_mu.append([isotope, tk.StringVar()])
            var_sig_mu[i][1].set(0.0)
            entr_sig_mu.append([isotope, var_sig_mu[i][1],
                                      tk.Entry(self.parent, textvariable=var_sig_mu[i][1])])
            entr_sig_mu[i][2].grid(row=self.row_id+i, column=self.column_id, sticky="nesw")
            #
            var_sig_std.append([isotope, tk.StringVar()])
            var_sig_std[i][1].set(0.0)
            entr_sig_std.append([isotope, var_sig_std[i][1],
                                      tk.Entry(self.parent, textvariable=var_sig_std[i][1])])
            entr_sig_std[i][2].grid(row=self.row_id+i, column=self.column_id+1, sticky="nesw")
            #
            i += 1
    #
    def create_simple_entry(self, var, command=None, text_default=0.0):
        var.set(text_default)
        entry = tk.Entry(self.parent, textvariable=var, fg=self.fg, bg=self.bg, highlightthickness=0)
        entry.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                   sticky="nesw")
        if command != None:
            entry.bind("<Return>", command)
        #
        return entry
    #
    def create_simple_entries(self, command=None, text_default=0.0):
        var = tk.StringVar()
        var.set(text_default)
        entry = tk.Entry(self.parent, textvariable=var, fg=self.fg, bg=self.bg, highlightthickness=0)
        entry.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                   sticky="nesw")
        if command != None:
            entry.bind("<Return>", command)
        #
        return var, entry
    #
    def create_entries_column(self, var_mu, entr_mu, var_std, entr_std, input_isotopes):
        i = 0
        for isotope in input_isotopes[:, 0]:
            var_mu.append([isotope, tk.StringVar()])
            var_mu[i][1].set(0.0)
            entr_mu.append([isotope, var_mu[i][1],
                                      tk.Entry(self.parent, textvariable=var_mu[i][1])])
            entr_mu[i][2].grid(row=self.row_id+i, column=self.column_id, sticky="nesw")
            #
            var_std.append([isotope, tk.StringVar()])
            var_std[i][1].set(0.0)
            entr_std.append([isotope, var_std[i][1],
                                      tk.Entry(self.parent, textvariable=var_std[i][1])])
            entr_std[i][2].grid(row=self.row_id+i, column=self.column_id+1, sticky="nesw")
            #
            i += 1
    #
    def create_option_files(self, type_name, file_selected, file_type, var_file_default, n_isotopes, command=None):
        if type_name == "STD":
            var_file_std_indiv = []
            file_helper = ["All Standards"]
            for file in file_type:
                parts = file.split("/")
                file_helper.append(parts[-1])
            parts_actual = file_selected.split("/")
            option_list_files_std = np.array(file_helper)
            for i in range(n_isotopes):
                var_file_std_indiv.append(tk.StringVar())
                if file_selected in file_type:
                    var_file_std_indiv[i].set(parts_actual[-1])
                else:
                    var_file_std_indiv[i].set(option_list_files_std[1])
        elif type_name == "SMPL":
            var_file_smpl_indiv = []
            file_helper = ["All Samples"]
            for file in file_type:
                parts = file.split("/")
                file_helper.append(parts[-1])
            parts_actual = file_selected.split("/")
            option_list_files_smpl = np.array(file_helper)
            for i in range(n_isotopes):
                var_file_smpl_indiv.append(tk.StringVar())
                if file_selected in file_type:
                    var_file_smpl_indiv[i].set(parts_actual[-1])
                else:
                    var_file_smpl_indiv[i].set(option_list_files_smpl[1])
        #
        parts_actual = file_selected.split("/")
        if file_selected in file_type:
                var_file_default.set(parts_actual[-1])
        else:
            if type_name == "STD":
                var_file_default.set(option_list_files_std[1])
            elif type_name == "SMPL":
                var_file_default.set(option_list_files_smpl[1])
        #
        if type_name == "STD":
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, var_file_default, *option_list_files_std)
            else:
                opt_menu = tk.OptionMenu(self.parent, var_file_default, *option_list_files_std,
                                                  command=command)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
            #
            return var_file_std_indiv, option_list_files_std
        elif type_name == "SMPL":
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, var_file_default, *option_list_files_smpl)
            else:
                opt_menu = tk.OptionMenu(self.parent, var_file_default, *option_list_files_smpl,
                                                  command=command)
            opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
            #
            return var_file_smpl_indiv, option_list_files_smpl
    #
    def create_option_files_column(self, var_file_indiv, option_list, n_isotopes, command=None):
        if command == None:
            for i in range(n_isotopes):
                opt_menu = tk.OptionMenu(self.parent, var_file_indiv[i], *option_list)
                opt_menu.grid(row=self.row_id+i, column=self.column_id, rowspan=1, columnspan=1, sticky="nesw")
        else:
            for i in range(n_isotopes):
                opt_menu = tk.OptionMenu(self.parent, var_file_indiv[i], *option_list, command=command)
                opt_menu.grid(row=self.row_id+i, column=self.column_id, rowspan=1, columnspan=1, sticky="nesw")
    #
    def create_radiobutton_column(self, var_rb, isotopes=None, list_data=None, command=None, relief=tk.FLAT):
        var_rb.set(0)
        try:
            n_isotopes = len(isotopes)
        except:
            n_isotopes = len(isotopes[:, 0])
        for i in range(n_isotopes):
            if command == None:
                if i == 0:
                    rb = tk.Radiobutton(self.parent, text="", variable=var_rb, value=0, relief=relief)
                else:
                    rb = tk.Radiobutton(self.parent, text="", variable=var_rb, value=i, relief=relief)
            else:
                rb = tk.Radiobutton(self.parent, text="", variable=var_rb, value=i, command=command, relief=relief)
            if self.n_rows == 1:
                rb.grid(row=self.row_id+i, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                        sticky="nesw")
            else:
                rb.grid(row=self.row_id+self.n_rows*i, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                        sticky="nesw")
    #
    def create_radiobutton(self, var_rb, value_rb, color_bg, relief=tk.FLAT, fg="black", command=None, text="",
                           font="sans 10 normal", sticky="nesw", anchor="w"):
        if value_rb == 0:
            var_rb.set(var_rb.get())
        if command == None:
            rb = tk.Radiobutton(
                self.parent, text=text, variable=var_rb, value=value_rb, bg=color_bg, fg=fg, activebackground=color_bg,
                relief=relief, selectcolor=color_bg, font=font, anchor=anchor)
        else:
            rb = tk.Radiobutton(
                self.parent, text=text, variable=var_rb, value=value_rb, bg=color_bg, fg=fg, activebackground=color_bg,
                relief=relief, selectcolor=color_bg, font=font, anchor=anchor, command=command)
        rb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                sticky=sticky)
        return rb
    #
    def create_simple_radiobutton(self, var_rb, value_rb, text, bg_active="white", fg_active="black", command=None,
                                  sticky="nesw", relief=tk.GROOVE, value_rb_default=0):
        if value_rb == value_rb_default:
            var_rb.set(value_rb_default)
        if command == None:
            rbtn = tk.Radiobutton(self.parent, text=text, variable=var_rb, value=value_rb, bg=self.bg, fg=self.fg,
                                activebackground=bg_active, activeforeground=fg_active, relief=relief,
                                  highlightthickness=0, font="sans 10 bold")
        else:
            rbtn = tk.Radiobutton(self.parent, text=text, variable=var_rb, value=value_rb, bg=self.bg, fg=self.fg,
                                activebackground=bg_active, activeforeground=fg_active, relief=relief,
                                  highlightthickness=0, command=command, font="sans 10 bold")
        rbtn.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                sticky=sticky)
        #
        return rbtn
    #
    def create_simple_checkbox(self, var_cb, text, command=None, set_sticky="nesw", own_color=False, var_anchor="w"):
        if own_color == False:
            color_rgb = matplotlib.colors.ColorConverter.to_rgb(self.bg)
            h, l, s = colorsys.rgb_to_hls(*color_rgb)
            color_light = matplotlib.colors.to_hex(colorsys.hls_to_rgb(h, min(1, l*1.5), s=s))
        else:
            color_light = self.fg
        if command == None:
            cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, fg="black", anchor=var_anchor,
                                highlightthickness=0, bd=0, activebackground=self.bg, activeforeground=self.fg,
                                selectcolor="white")
        else:
            cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, fg="black", anchor=var_anchor,
                                highlightthickness=0, bd=0, activebackground=self.bg, activeforeground=self.fg,
                                selectcolor="white", command=command)
        #
        if set_sticky == "":
            cb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns)
        else:
            cb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                    sticky=set_sticky)
        #
        return cb
    #
    def create_checkbox(self, var_cb, text, type_name="STD", relief=tk.GROOVE, command=None):
        if type_name == "STD":
            if command == None:
                cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, relief=relief)
            else:
                cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, relief=relief, command=command)
            cb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                    sticky="nesw")
        elif type_name == "SMPL":
            if command == None:
                cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, relief=relief)
            else:
                cb = tk.Checkbutton(self.parent, text=text, variable=var_cb, bg=self.bg, relief=relief, command=command)
            cb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                    sticky="nesw")
        return cb
    #
    def create_option_srm(self, var_srm, text_set, command=None, option_list=None, fg_active="black", bg_active="white",
                          sort_list=True):
        #
        var_srm.set(text_set)
        if option_list == None:
            option_list_srm = np.array([
                ["NIST 606"], ["NIST 610"], ["NIST 610 (GeoReM)"], ["NIST 610 (Spandler)"], ["NIST 611"],
                ["NIST 611 (GeoReM)"], ["NIST 612"], ["NIST 612 (GeoReM)"], ["NIST 613"], ["NIST 613 (GeoReM)"],
                ["NIST 614"], ["NIST 614 (GeoReM)"], ["NIST 615"], ["NIST 615 (GeoReM)"], ["NIST 616"],
                ["NIST 616 (GeoReM)"], ["NIST 617"], ["NIST 617 (GeoReM)"], ["USGS BCR-2G (GeoReM)"],
                ["USGS GSD-1G (GeoReM)"], ["USGS GSE-1G (GeoReM)"], ["B6"], ["Durango Apatite"], ["Scapolite 17"],
                ["BAM-376"], ["BCR-2G"], ["BL-Q"], ["Br-Glass"], ["GSD-1G (GeoReM)"], ["GSE-1G (GeoReM)"], ["GSE-2G"],
                ["HAL-O"], ["K-Br"], ["MACS-3"], ["Po 724"], ["STDGL-2B2"]])[:, 0]
        else:
            option_list_srm = option_list

        if sort_list == True:
            option_list_srm.sort()
        #
        if command == None:
            opt_menu_srm = tk.OptionMenu(self.parent, var_srm, *option_list_srm)
        else:
            opt_menu_srm = tk.OptionMenu(self.parent, var_srm, *option_list_srm, command=command)
        opt_menu_srm.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
        opt_menu_srm.config(fg=self.fg, bg=self.bg, activebackground=self.bg, activeforeground=self.fg,
                            highlightthickness=0)
        opt_menu_srm["menu"].config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active)
        #
        return opt_menu_srm
    #
    def create_option_mineral(self, var_min, text_set, command=None, option_list=None, fg_active="black", bg_active="white"):
        #
        var_min.set(text_set)
        if option_list == None:
            option_list_min = ["Quartz", "Calcite", "Fluorite", "Apatite-Cl", "Apatite-F", "Apatite-OH", "Forsterite",
                               "Fayalite", "Tephroite", "Albite", "Anorthite", "Orthoclase", "Microcline"]
        else:
            option_list_min = option_list
        option_list_min.sort()
        if command == None:
            opt_menu_min = tk.OptionMenu(self.parent, var_min, *option_list_min)
        else:
            opt_menu_min = tk.OptionMenu(self.parent, var_min, *option_list_min, command=command)
        opt_menu_min.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
        opt_menu_min.config(fg=self.fg, bg=self.bg, activebackground=self.bg, activeforeground=self.fg,
                            highlightthickness=0)
        opt_menu_min["menu"].config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active)
        #
        return opt_menu_min
    #
    def create_option_isotope(self, var_iso, text_set, command=None, option_list=None, fg_active="black", bg_active="white"):
        var_iso.set(text_set)
        if option_list == None:
            option_list_iso = ["Si29"]
        else:
            option_list_iso = option_list
        if command == None:
            opt_menu_min = tk.OptionMenu(self.parent, var_iso, *option_list_iso)
        else:
            opt_menu_min = tk.OptionMenu(self.parent, var_iso, *option_list_iso, command=command)
        opt_menu_min.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
        opt_menu_min.config(fg=self.fg, bg=self.bg, activebackground=self.bg, activeforeground=self.fg,
                            highlightthickness=0)
        opt_menu_min["menu"].config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active)
        #
        return opt_menu_min
    #
    def create_option_menu(self, var_opt, text_set, command=None, option_list=None, fg_active="black", bg_active="white"):
        #
        var_opt.set(text_set)
        #
        if option_list == None:
            option_list_opt = ["Si29"]
        else:
            option_list_opt = option_list
        #
        if command == None:
            opt_menu = tk.OptionMenu(self.parent, var_opt, text_set, *option_list_opt)
        else:
            opt_menu = tk.OptionMenu(self.parent, var_opt, text_set, *option_list_opt, command=command)
        #
        opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
        opt_menu.config(fg=self.fg, bg=self.bg, activebackground=self.bg, activeforeground=self.fg,
                        highlightthickness=0)
        opt_menu["menu"].config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active)
        #
        return opt_menu
    #
    def create_option_is(self, var_is, var_is_default, isotopes, is_container=None, default_is=True, command=None):
        option_list_is = isotopes[:, 0]
        if default_is == True:
            var_is_default.set("Select Isotope")
            if command == None:
                opt_menu_is = tk.OptionMenu(self.parent, var_is_default, *option_list_is)
            else:
                opt_menu_is = tk.OptionMenu(self.parent, var_is_default, *option_list_is, command=command)
            opt_menu_is.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                             sticky="nesw")
        else:
            var_is.set("Select Isotope")
            if command == None:
                opt_menu_is = tk.OptionMenu(self.parent, var_is, *option_list_is)
            else:
                opt_menu_is = tk.OptionMenu(self.parent, var_is, *option_list_is, command=command)
            is_container.append(opt_menu_is)
            opt_menu_is.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                             sticky="nesw")
    #
    def create_option_is_column(self, is_container, isotopes, command=None):
        option_list_is = isotopes[:, 0]
        for i in range(len(isotopes[:, 0])):
            is_container.append(tk.StringVar())
            is_container[i].set("Select isotope")
            if command == None:
                opt_menu_is = tk.OptionMenu(self.parent, is_container[i], *option_list_is)
            else:
                opt_menu_is = tk.OptionMenu(self.parent, is_container[i], *option_list_is, command=lambda variable=is_container[i], index=i: Essentials(variable).change_option(index))
            opt_menu_is.grid(row=self.row_id+i, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                             sticky="nesw")
    #
    def create_option_srm_default(self, var_srm_default, found_srm, command=None, conc=False):
        option_list_srm = found_srm
        if conc == False:
            var_srm_default.set("Select SRM")
        else:
            var_srm_default.set(option_list_srm[0])
        if command == None:
            opt_menu_srm = tk.OptionMenu(self.parent, var_srm_default, *option_list_srm)
        else:
            opt_menu_srm = tk.OptionMenu(self.parent, var_srm_default, *option_list_srm, command=command)
        opt_menu_srm.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                         sticky="nesw")
    #
    def create_option_is_default(self, var_is_default, found_is, command=None):
        option_list_is = found_is
        var_is_default.set("Select SRM")
        if command == None:
            opt_menu_is = tk.OptionMenu(self.parent, var_is_default, *option_list_is)
        else:
            opt_menu_is = tk.OptionMenu(self.parent, var_is_default, *option_list_is, command=command)
        opt_menu_is.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                         sticky="nesw")
    #
    def create_option_srm_column(self, srm_container, found_srm, n_isotopes, command=None):
        option_list_srm = found_srm
        for i in range(n_isotopes):
            srm_container.append(tk.StringVar())
            srm_container[i].set("Select SRM")
            if command == None:
                opt_menu = tk.OptionMenu(self.parent, srm_container[i], *option_list_srm)
            else:
                opt_menu = tk.OptionMenu(self.parent, srm_container[i], *option_list_srm, command=lambda variable=srm_container[i], index=i: Essentials(variable).change_option(index))
            opt_menu.grid(row=self.row_id+i, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                             sticky="nesw")
    #
    def create_option_file(self, found_files, var_file, command=None):
        option_list = []
        for file in found_files:
            parts = file.split("/")
            option_list.append([parts[-1]])
        option_list_srm = option_list
        var_file.set("Select File")
        if command == None:
            opt_menu = tk.OptionMenu(self.parent, var_file, *option_list_srm)
        else:
            opt_menu = tk.OptionMenu(self.parent, var_file, *option_list_srm, command=command)
        opt_menu.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
    #
    def create_simple_optionmenu(self, var_opt, var_default, var_list, fg_active=None, bg_active=None, command=None):
        var_opt.set(var_default)
        if command == None:
            opt_menu = tk.OptionMenu(self.parent, var_opt, *var_list)
        else:
            opt_menu = tk.OptionMenu(self.parent, var_opt, *var_list, command=command)
        #
        if fg_active == None:
            fg_active = self.fg
        if bg_active == None:
            bg_active == self.bg
        #
        opt_menu["menu"].config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active)
        opt_menu.config(fg=self.fg, bg=self.bg, activeforeground=fg_active, activebackground=bg_active,
                        highlightthickness=0)
        #
        opt_menu.grid(
            row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns, sticky="nesw")
        #
        return opt_menu
    #
    def create_listbox(self, val_width, val_height):
        frame_lb = tk.Frame(self.parent, bg=self.bg)
        frame_lb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
        #
        scrollbar_y = tk.Scrollbar(frame_lb, orient="vertical")
        scrollbar_x = tk.Scrollbar(frame_lb, orient="horizontal")
        listbox = tk.Listbox(frame_lb, bg=self.bg, yscrollcommand=scrollbar_y.set, xscrollcommand=scrollbar_x.set)
        #
        scrollbar_y.config(command=listbox.yview)
        scrollbar_x.config(command=listbox.xview)
        scrollbar_x.pack(side="bottom", fill="x", pady=0)
        listbox.pack(side="left", fill="both", padx=0, pady=0, expand=True)
        scrollbar_y.pack(side="right", fill="y", padx=0)
        frame_lb.columnconfigure(self.column_id, weight=1)
        frame_lb.rowconfigure(self.row_id, weight=1)
        #
        return listbox
    #
    def create_simple_listbox(self, include_scrb_x=True):
        frame_lb = tk.Frame(self.parent, bg=self.bg)
        frame_lb.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                      sticky="nesw")
        #
        scrollbar_y = tk.Scrollbar(frame_lb, orient="vertical")
        if include_scrb_x == True:
            scrollbar_x = tk.Scrollbar(frame_lb, orient="horizontal")
            listbox = tk.Listbox(
                frame_lb, fg=self.fg, bg=self.bg, yscrollcommand=scrollbar_y.set, xscrollcommand=scrollbar_x.set)
        else:
            listbox = tk.Listbox(frame_lb, fg=self.fg, bg=self.bg, yscrollcommand=scrollbar_y.set)
        #
        scrollbar_y.config(command=listbox.yview)
        if include_scrb_x == True:
            scrollbar_x.config(command=listbox.xview)
            scrollbar_x.pack(side="bottom", fill="x", pady=0)
        listbox.pack(side="left", fill="both", padx=0, pady=0, expand=True)
        scrollbar_y.pack(side="right", fill="y", padx=0)
        frame_lb.columnconfigure(self.column_id, weight=1)
        frame_lb.rowconfigure(self.row_id, weight=1)
        #
        return listbox
    #
    def create_simple_listbox_grid(self, include_scrb_x=True):
        scrollbar_y = tk.Scrollbar(self.parent, orient="vertical")
        if include_scrb_x == True:
            scrollbar_x = tk.Scrollbar(self.parent, orient="horizontal")
            listbox = tk.Listbox(
                self.parent, bg=self.bg, fg=self.fg, yscrollcommand=scrollbar_y.set, xscrollcommand=scrollbar_x.set)
        else:
            listbox = tk.Listbox(self.parent, bg=self.bg, fg=self.fg, yscrollcommand=scrollbar_y.set)
        #
        scrollbar_y.config(command=listbox.yview)
        if include_scrb_x == True:
            scrollbar_x.config(command=listbox.xview)
            scrollbar_x.grid(row=self.row_id+self.n_rows-1, column=self.column_id, rowspan=1, columnspan=self.n_columns,
                             sticky="ew")
            listbox.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows-1, columnspan=self.n_columns-1,
                         sticky="nesw")
        else:
            listbox.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns - 1,
                         sticky="nesw")
        scrollbar_y.grid(row=self.row_id, column=self.column_id+self.n_columns-1, rowspan=self.n_rows, columnspan=1,
                         sticky="ns")
        #
        if include_scrb_x == True:
            return listbox, scrollbar_x, scrollbar_y
        else:
            return listbox, scrollbar_y
    #
    def create_treeview(self, n_categories=2, text_1="Ratio", text_2="\u03BC", width_1 = 90, width_2 = 90, text_n=[],
                        width_n=[], individual=False):
        ttk.Style().configure("Treeview", background=self.bg, foreground=self.fg, fieldbackground=self.bg)
        style = ttk.Style()
        style.configure("Treeview.Heading", background=self.bg, pressed_color=self.bg,
                        highlight_color=self.bg, foreground=self.fg)
        #
        if n_categories == 2 and individual == False:
            columns = ("#1", "#2")
            treeview = ttk.Treeview(self.parent, columns=columns, show="headings")
            treeview.heading("#1", text=text_1)
            treeview.column("#1", minwidth=0, width=width_1, stretch=tk.NO, anchor=tk.CENTER)
            treeview.heading("#2", text=text_2)
            treeview.column("#2", minwidth=0, width=width_2, stretch=tk.YES, anchor=tk.CENTER)
            treeview.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
        #
        if n_categories > 1 and individual == True:
            columns = []
            for n in range(n_categories):
                var_i = "#" + str(n + 1)
                columns.append(var_i)
            #
            treeview = ttk.Treeview(self.parent, columns=columns, show="headings")
            #
            for index, element in enumerate(columns):
                treeview.heading(element, text=text_n[index])
                treeview.column(element, minwidth=0, width=width_n[index], stretch=tk.NO, anchor=tk.CENTER)
            #
            treeview.grid(row=self.row_id, column=self.column_id, rowspan=self.n_rows, columnspan=self.n_columns,
                          sticky="nesw")
        #
        return treeview


#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# results.py
# Maximilian Beeskow
# 16.08.2021
# ----------------------
#
## MODULES
import numpy as np
import os
from modules import data
from modules.chemistry import PeriodicSystemOfElements as PSE
from modules.standard import StandardReferenceMaterials as SRM
import re
import matplotlib.colors as mplcol
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg, NavigationToolbar2Tk)
from scipy import stats
import pandas as pd
import tkinter as tk
from tkinter import ttk, font
import tkinter.filedialog as fd
#
## CLASSES
class Sensitivities:
    #
    def __int__(self, signal, concentration, internal_std, isotopes, limits):
        self.signal = signal
        self.concentration = concentration
        self.internal_std = internal_std
        self.isotopes = isotopes
        self.limits = limits
    #
    def calculate_xi(self):
        xi = []
        for isotope in self.isotopes:
            signal_i = np.mean(self.signal[isotope][self.limits[0]:self.limits[1]])
            signal_is = np.mean(self.signal[self.internal_std][self.limits[0]:self.limits[1]])
            concentration_i = self.concentration[isotope]
            concentration_is = self.concentration[self.internal_std]
            xi.append([isotope, (signal_i)/(signal_is)*(concentration_is)/(concentration_i)])
        #
        return xi
#
class IsotopeRatios:
    #
    def __init__(self, parent, color_bg, signal, isotopes):
        # Input
        self.parent = parent
        self.color_bg = color_bg
        self.signal = signal
        self.isotopes = isotopes
        self.color_fg = "black"
        #
        padx_value = 0
        pady_value = 0
        ipadx_value = 1
        ipady_value = 1
        #
        self.ir_window = tk.Toplevel(self.parent)
        self.ir_window.geometry("900x700")
        self.ir_window.title("Isotope Ratios")
        #
        for x in range(12):
            tk.Grid.columnconfigure(self.ir_window, x, weight=1)
        for y in range(27):
            tk.Grid.rowconfigure(self.ir_window, y, weight=1)
        #
        self.ir_window.grid_columnconfigure(0, minsize=80)
        self.ir_window.grid_columnconfigure(1, minsize=65)
        self.ir_window.grid_columnconfigure(2, minsize=80)
        self.ir_window.grid_columnconfigure(3, minsize=60)
        for i in range(4, 12):
            self.ir_window.grid_columnconfigure(i, minsize=75)
        for i in range(0, 2):
            self.ir_window.grid_rowconfigure(i, minsize=35)
        self.ir_window.grid_rowconfigure(2, minsize=15)
        for i in range(3, 26):
            self.ir_window.grid_rowconfigure(i, minsize=25)
        self.ir_window.grid_rowconfigure(26, minsize=40)
        #
        # Labels
        lbl_01 = tk.Label(self.ir_window, text="Select Isotope")
        lbl_01.grid(row=0, column=0, columnspan=2, padx=padx_value, pady=pady_value,
                    ipadx=ipadx_value, ipady=ipady_value, sticky="nesw")
        #
        # Options menus
        option_list = self.isotopes
        self.var_srm = tk.StringVar()
        self.var_srm.set(option_list[0])
        opt_menu = tk.OptionMenu(self.ir_window, self.var_srm, *option_list, command=self.option_changed)
        opt_menu.grid(row=1, column=0, columnspan=2, padx=padx_value, pady=pady_value,
                      ipadx=ipadx_value, ipady=ipady_value, sticky="nesw")
        #
    def option_changed(self, op):
        #
        try:
            self.fig.clf()
            self.ax.cla()
            self.canvas.get_tk_widget().pack_forget()
            self.fig_histo.clf()
            self.ax_histo.cla()
            self.canvas_histo.get_tk_widget().pack_forget()
        except AttributeError:
            pass

        try:
            if self.canvas:
                self.canvas.destroy()
            if self.canvas_histo:
                self.canvas.destroy()
        except AttributeError:
            pass

        self.canvas = None
        self.canvas_histo = None
        #
        ioi = self.var_srm.get() # ioi = isotope-of-interest
        mean_ioi = np.mean(self.signal[ioi])
        n_isotopes = len(self.isotopes)
        n_left = int(n_isotopes/2)
        self.ratios = []
        variables = []
        entries = []
        for i in range(n_left): # left column
            tk.Label(self.ir_window, text=str(self.isotopes[i][0])+"/"+str(ioi)).grid(row=i+3, column=0, sticky="nesw")
            value = round(np.mean(self.signal[self.isotopes[i][0]])/mean_ioi, 4)
            self.ratios.append(value)
            variables.append(tk.StringVar())
            entries.append(tk.Entry(self.ir_window, textvariable=variables[-1]))
            entries[-1].grid(row=i+3, column=1, sticky="nesw")
            entries[-1].insert(0, value)
        for i in range(n_left, n_isotopes): # right column
            tk.Label(self.ir_window, text=str(self.isotopes[i][0])+"/"+str(ioi)).grid(row=i-8, column=2, sticky="nesw")
            value = round(np.mean(self.signal[self.isotopes[i][0]])/mean_ioi, 4)
            self.ratios.append(value)
            variables.append(tk.StringVar())
            entries.append(tk.Entry(self.ir_window, textvariable=variables[-1]))
            entries[-1].grid(row=i-8, column=3, sticky="nesw")
            entries[-1].insert(0, value)
        #
        X = self.isotopes[:, 0]
        Y = self.ratios
        #
        self.fig = Figure(figsize=(10, 5), facecolor=self.color_bg)
        self.ax = self.fig.add_subplot()
        #
        self.ax.axhline(1.0, color="tomato", linestyle="dashed")
        self.ax.bar(X, Y)
        self.ax.set_xticklabels(X, rotation=90)
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_axisbelow(True)
        self.ax.set_ylabel("Isotope ratio (*/"+str(ioi)+")", labelpad=0.5)

        self.plotting_area = tk.Frame(self.ir_window, bg=self.color_bg)
        self.plotting_area.grid(row=0, column=4, rowspan=26, columnspan=8, sticky="nesw")
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.plotting_area)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        #
        self.plotting_histo = tk.Frame(self.ir_window, bg=self.color_bg)
        self.plotting_histo.grid(row=n_left+3, column=0, rowspan=int(26-n_left), columnspan=4, sticky="nesw")
        self.fig_histo = Figure(figsize=(10, 5), facecolor=self.color_bg, tight_layout=True)
        self.ax_histo = self.fig_histo.add_subplot()
        #
        self.ax_histo.hist(Y, bins=75)
        self.ax_histo.grid(True)
        self.ax_histo.set_axisbelow(True)
        self.ax_histo.set_xlabel("Isotope ratio (*/"+str(ioi)+")", labelpad=0.5)
        self.ax_histo.set_ylabel("Frequency", labelpad=0.5)

        self.canvas_histo = FigureCanvasTkAgg(self.fig_histo, master=self.plotting_histo)
        self.canvas_histo.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        #
class SimpleSignals:
    #
    def __init__(self, parent, color_bg, signal, isotopes, times, times_def, times_def_bg, files_std, files_smpl, file_actual):
        self.parent = parent
        self.color_bg = color_bg
        self.color_fg = "black"
        self.signal = signal
        self.isotopes = isotopes
        self.times = times
        self.times_def = times_def
        self.times_def_bg = times_def_bg
        self.files_std = files_std
        self.files_smpl = files_smpl
        self.file_actual = file_actual
        #
        self.n_isotopes = len(self.isotopes[:, 0])
        #
        self.index_start_sig = 0
        self.index_end_sig = len(self.times)-1
        self.indices_sig = [self.index_start_sig, self.index_end_sig]
        self.index_start_bg = 0
        self.index_end_bg = len(self.times)-1
        self.indices_bg = [self.index_start_bg, self.index_end_bg]
        #
    #
    def calculate_sensitivities(self):
        self.xi_window = tk.Toplevel(self.parent)
        self.xi_window.geometry("900x800")
        self.xi_window.title("Sensitivities")
        #
        self.xi_values = []
        #
        self.pse_list = PSE().get_element_names()
        #
        for x in range(12):
            tk.Grid.columnconfigure(self.xi_window, x, weight=1)
        for y in range(26):
            tk.Grid.rowconfigure(self.xi_window, y, weight=1)
        #
        self.xi_window.grid_columnconfigure(0, minsize=90)
        self.xi_window.grid_columnconfigure(1, minsize=80)
        self.xi_window.grid_columnconfigure(2, minsize=90)
        self.xi_window.grid_columnconfigure(3, minsize=80)
        for i in range(4, 12):
            self.xi_window.grid_columnconfigure(i, minsize=70)
        for i in range(0, 4):
            self.xi_window.grid_rowconfigure(i, minsize=35)
        for i in range(4, 26):
            self.xi_window.grid_rowconfigure(i, minsize=30)
        #
        # Labels
        lbl_01 = tk.Label(self.xi_window, text="Internal Standard")
        lbl_01.grid(row=0, column=0, columnspan=2, sticky="nesw")
        lbl_02 = tk.Label(self.xi_window, text="Time Interval")
        lbl_02.grid(row=0, column=2, columnspan=2, sticky="nesw")
        lbl_03 = tk.Label(self.xi_window, text="Start")
        lbl_03.grid(row=1, column=2, sticky="nesw")
        lbl_04 = tk.Label(self.xi_window, text="End")
        lbl_04.grid(row=2, column=2, sticky="nesw")
        #
        self.srm_values = []
        option_list_srm = np.array([["NIST 606"], ["NIST 610"], ["NIST 610 (GeoReM)"], ["NIST 610 (Spandler)"],
                                     ["NIST 611"], ["NIST 611 (GeoReM)"], ["NIST 612"], ["NIST 612 (GeoReM)"],
                                     ["NIST 613"], ["NIST 613 (GeoReM)"], ["NIST 614"], ["NIST 614 (GeoReM)"],
                                     ["NIST 615"], ["NIST 615 (GeoReM)"], ["NIST 616"], ["NIST 616 (GeoReM)"],
                                     ["NIST 617"], ["NIST 617 (GeoReM)"], ["USGS BCR-2G (GeoReM)"],
                                     ["USGS GSD-1G (GeoReM)"], ["USGS GSE-1G (GeoReM)"]])[:, 0]
        self.var_srm = tk.StringVar()
        self.var_srm.set("Select SRM")
        opt_menu_srm = tk.OptionMenu(self.xi_window, self.var_srm, *option_list_srm, command=self.option_changed_srm)
        opt_menu_srm.grid(row=2, column=0, columnspan=2, sticky="nesw")
        #
        # Options menus (Integration window)
        option_list_iw = []
        if len(self.times_def) == 0:
            option_list_iw.append(["No Time Intervals"])
        else:
            for i in range(len(self.times_def)):
                option_list_iw.append([str(self.times_def[i][0])+" - "+str(self.times_def[i][1])])
        option_list_iw = np.array(option_list_iw)[:, 0]
        self.var_iw = tk.StringVar()
        self.var_iw.set("Select Time Interval")
        opt_menu_iw = tk.OptionMenu(self.xi_window, self.var_iw, *option_list_iw,
                                    command=lambda op, parent_plot=self.xi_window:
                                    self.selected_integration_window(parent_plot, op))
        opt_menu_iw.grid(row=3, column=2, columnspan=2, sticky="nesw")
        #
        # Treeviews
        self.xi_treeview = self.make_treeview_xi(parent=self.xi_window)
        self.srm_treeview = self.make_treeview_srm(parent=self.xi_window)
        #
        # Entries Time Interval
        self.index_start = 0
        self.index_end = len(self.times)-1
        self.indices_sig = [self.index_start, self.index_end]

        self.t_start = tk.StringVar()
        self.t_start.set(self.times.iloc[0])
        self.entr_t_start = tk.Entry(self.xi_window, textvariable=self.t_start)
        self.entr_t_start.grid(row=1, column=3, sticky="nesw")
        self.entr_t_start.bind("<Return>", lambda event, entr=self.entr_t_start, var=self.t_start, t_id=self.indices_sig[0],
                                                  pos="start", parent_treeview=self.xi_treeview,
                                                  parent_plot=self.xi_window, key_plot=True: self.find_nearest_time(entr, var, t_id,
                                                                                                     pos, parent_treeview,
                                                                                                     parent_plot, key_plot, event))
        self.t_end = tk.StringVar()
        self.t_end.set(self.times.iloc[-1])
        self.entr_t_end = tk.Entry(self.xi_window, textvariable=self.t_end)
        self.entr_t_end.grid(row=2, column=3, sticky="nesw")
        self.entr_t_end.bind("<Return>", lambda event, entr=self.entr_t_end, var=self.t_end, t_id=self.indices_sig[1],
                                                pos="end", parent_treeview=self.xi_treeview,
                                                parent_plot=self.xi_window, key_plot=True: self.find_nearest_time(entr, var, t_id,
                                                                                                     pos, parent_treeview,
                                                                                                     parent_plot, key_plot, event))
        #
        # Options menus (SRM)
        option_list = self.isotopes[:, 0]
        self.var_is = tk.StringVar()
        self.var_is.set("Select Isotope")
        opt_menu = tk.OptionMenu(self.xi_window, self.var_is, *option_list,
                                 command=lambda op, parent_treeview=self.xi_treeview, parent_plot=self.xi_window:
                                 self.option_changed(parent_treeview, parent_plot, op))
        opt_menu.grid(row=1, column=0, columnspan=2, sticky="nesw")
        #
        # Buttons
        btn_01 = tk.Button(self.xi_window, text="Load SRM", fg=self.color_fg, bg=self.color_bg,
                           command=lambda parent=self.xi_window: self.load_srm_from_csv(parent))
        btn_01.grid(row=3, column=0, columnspan=2, sticky="nesw")
    #
    def make_treeview_xi(self, parent):
        # General Settings
        ttk.Style().configure("Treeview", background=self.color_bg, foreground="black",
                              fieldbackground=self.color_bg)
        my_font = font.Font(size="11", weight="normal")
        style = ttk.Style()
        style.configure("Treeview.Heading", font=my_font, background="gray", pressed_color="gray",
                        highlight_color="gray", foreground=self.color_fg)
        style.map("Treeview.Heading", background = [("pressed", "!focus", "gray"),
                                                    ("active", "gray"), ("disabled", self.color_bg)])
        #
        columns = ("#1", "#2", "#3")
        treeview = ttk.Treeview(parent, columns=columns, show="headings")
        treeview.heading("#1", text="Isotope")
        treeview.column("#1", minwidth=0, width=90, stretch=tk.NO)
        treeview.heading("#2", text="Sensitivity")
        treeview.column("#2", minwidth=0, width=85, stretch=tk.NO)
        treeview.heading("#3", text="Error")
        treeview.column("#3", minwidth=0, width=85, stretch=tk.NO)
        treeview.grid(row=5, column=0, rowspan=10, columnspan=3, sticky="nesw")
        #
        return treeview
    #
    def make_treeview_srm(self, parent, row_id=16, column_id=0, n_rows=10, n_columns=3):
        # General Settings
        ttk.Style().configure("Treeview", background=self.color_bg, foreground="black",
                              fieldbackground=self.color_bg)
        my_font = font.Font(size="11", weight="normal")
        style = ttk.Style()
        style.configure("Treeview.Heading", font=my_font, background="gray", pressed_color="gray",
                        highlight_color="gray", foreground=self.color_fg)
        style.map("Treeview.Heading", background = [("pressed", "!focus", "gray"),
                                                    ("active", "gray"), ("disabled", self.color_bg)])
        #
        columns = ("#1", "#2")
        treeview = ttk.Treeview(parent, columns=columns, show="headings")
        treeview.heading("#1", text="Element")
        treeview.column("#1", minwidth=0, width=90, stretch=tk.NO)
        treeview.heading("#2", text="Concentration")
        treeview.column("#2", minwidth=0, width=170, stretch=tk.NO)
        treeview.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
        #
        return treeview
    #
    def selected_integration_window(self, parent_plot, op, mode_xi=True, key_plot=True, mode="SIG"):
        #
        if mode_xi == True:
            for i in self.xi_treeview.get_children():
                self.xi_treeview.delete(i)
            #
            if self.var_iw.get() != "No Time Intervals":
                key = re.search("(\d+\.\d+)"+" - "+"(\d+\.\d+)", self.var_iw.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                self.indices_sig[0] = t_id_start
                self.indices_sig[1] = t_id_end
                self.entr_t_start.delete(0, tk.END)
                self.entr_t_start.insert(0, t_start)
                self.entr_t_end.delete(0, tk.END)
                self.entr_t_end.insert(0, t_end)
                #
                xi_results = self.calculate_xi(id_start=self.indices_sig[0], id_end=self.indices_sig[1])
                for isotopes, xi_mean, xi_std in xi_results:
                    self.xi_treeview.insert("", tk.END, values=[isotopes, round(xi_mean, 4), round(xi_std, 4)])
                #
                if key_plot == True:
                    X = self.isotopes[:, 0]
                    xi_results = np.array(xi_results, dtype=object)
                    Y = np.array(xi_results)[:, 1]
                    #
                    self.make_bar_plot(X=X, Y=Y, parent=parent_plot)
                    self.make_histo_plot(X=X, Y=Y, parent=parent_plot)
            else:
                pass
        else:
            #
            if mode == "SIG":
                if self.var_iw_sig.get() != "No Time Intervals":
                    key = re.search("(\d+\.\d+)" + " - " + "(\d+\.\d+)", self.var_iw_sig.get())
                    t_start = float(key.group(1))
                    t_end = float(key.group(2))
                    #
                    t_id_start = self.times[self.times == t_start].index[0]
                    t_id_end = self.times[self.times == t_end].index[0]
                    #
                    self.indices_sig[0] = t_id_start
                    self.indices_sig[1] = t_id_end
                    self.entr_t_start_sig.delete(0, tk.END)
                    self.entr_t_start_sig.insert(0, t_start)
                    self.entr_t_end_sig.delete(0, tk.END)
                    self.entr_t_end_sig.insert(0, t_end)
                    #
                    for i in range(self.n_isotopes):
                        self.calculate_xi_2(var_file=self.var_file_std_default.get(), var_i=self.isotopes[:, 0][i],
                                            var_is=self.var_is_default.get(), index=i)
                        self.calculate_ci(var_file_std=self.var_file_std_default.get(),
                                          var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                                          var_i=self.isotopes[:, 0][i], index=i)
                        self.calculate_rsf(var_file_std=self.var_file_std_default.get(),
                                           var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                                           var_i=self.isotopes[:, 0][i], index=i)
                        self.calculate_lod(var_file_std=self.var_file_std_default.get(), var_file_smpl=self.var_file_smpl_default.get(),
                               var_is=self.var_is_default.get(), var_i=self.isotopes[:, 0][i], index=i)
                else:
                    pass
            else:
                if self.var_iw_bg.get() != "No Time Intervals":
                    key = re.search("(\d+\.\d+)" + " - " + "(\d+\.\d+)", self.var_iw_bg.get())
                    t_start = float(key.group(1))
                    t_end = float(key.group(2))
                    #
                    t_id_start = self.times[self.times == t_start].index[0]
                    t_id_end = self.times[self.times == t_end].index[0]
                    #
                    self.indices_bg[0] = t_id_start
                    self.indices_bg[1] = t_id_end
                    self.entr_t_start_bg.delete(0, tk.END)
                    self.entr_t_start_bg.insert(0, t_start)
                    self.entr_t_end_bg.delete(0, tk.END)
                    self.entr_t_end_bg.insert(0, t_end)
                    #
                    for i in range(self.n_isotopes):
                        self.calculate_xi_2(var_file=self.var_file_std_default.get(), var_i=self.isotopes[:, 0][i],
                                            var_is=self.var_is_default.get(), index=i)
                        self.calculate_ci(var_file_std=self.var_file_std_default.get(),
                                          var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                                          var_i=self.isotopes[:, 0][i], index=i)
                        self.calculate_rsf(var_file_std=self.var_file_std_default.get(),
                                           var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                                           var_i=self.isotopes[:, 0][i], index=i)
                        self.calculate_lod(var_file_std=self.var_file_std_default.get(), var_file_smpl=self.var_file_smpl_default.get(),
                               var_is=self.var_is_default.get(), var_i=self.isotopes[:, 0][i], index=i)
                else:
                    pass
    #
    def find_nearest_time(self, entr, var, t_id, pos, parent_treeview, parent_plot, key_plot, event, t_iw=None):
        #
        if t_iw == None:
            t_nearest = min(self.times, key=lambda x: abs(x - float(var.get())))
            entr.delete(0, tk.END)
            entr.insert(0, t_nearest)
            t_id = self.times[self.times == t_nearest].index[0]
            #
            for i in parent_treeview.get_children():
                parent_treeview.delete(i)
            #
            if pos == "start":
                self.indices_sig[0] = t_id
            else:
                self.indices_sig[1] = t_id
        else:
            t_id_start = self.times[self.times == t_iw[0]].index[0]
            t_id_end = self.times[self.times == t_iw[1]].index[0]
            self.indices_sig[0] = t_id_start
            self.indices_sig[1] = t_id_end
            self.entr_t_start.delete(0, tk.END)
            entr.insert(0, t_iw[0])
            self.entr_t_end.delete(0, tk.END)
            entr.insert(0, t_iw[1])
        #
        xi_results = self.calculate_xi(id_start=self.indices_sig[0], id_end=self.indices_sig[1])
        for isotopes, xi_mean, xi_std in xi_results:
            parent_treeview.insert("", tk.END, values=[isotopes, round(xi_mean, 4), round(xi_std, 4)])
        #
        if key_plot == True:
            X = self.isotopes[:, 0]
            xi_results = np.array(xi_results, dtype=object)
            Y = np.array(xi_results)[:, 1]
            #
            self.make_bar_plot(X=X, Y=Y, parent=parent_plot)
            self.make_histo_plot(X=X, Y=Y, parent=parent_plot)
    #
    def calculate_xi(self, id_start, id_end, var=False):
        isotopes = self.isotopes[:, 0]
        if var == False:
            internal_standard = self.var_is.get()
        else:
            internal_standard = var.get()
        key_is = re.search("(\D+)(\d+)", internal_standard)
        index_is = self.pse_list.index(key_is.group(1))
        #
        xi_helper = []
        xi_results = []
        for isotope in isotopes:
            xi_helper.clear()
            key = re.search("(\D+)(\d+)", isotope)
            index_i = self.pse_list.index(key.group(1))
            for i in range(id_start, id_end+1):
                if self.signal[isotope][i] > 0 and self.signal[internal_standard][i] > 0 and self.srm_values[index_i] > 0:
                    signal_i = self.signal[isotope][i]
                    signal_is = self.signal[internal_standard][i]
                    conc_ratio = self.srm_values[index_is]/self.srm_values[index_i]
                    xi = signal_i/signal_is * conc_ratio
                    xi_helper.append(xi)
                elif self.srm_values[index_i] == 0:
                    xi_helper.append(0.0)
            xi_results.append([isotope, np.mean(xi_helper), np.std(xi_helper, ddof=1)])
        #
        return xi_results
    #
    def option_changed(self, parent_treeview, parent_plot, op, category="sensitivity", key_plot=True):
        xi_results = self.calculate_xi(id_start=self.indices_sig[0], id_end=self.indices_sig[1])
        #
        for i in parent_treeview.get_children():
            parent_treeview.delete(i)
        #
        i = 0
        for isotopes, xi_mean, xi_std in xi_results:
            if category == "sensitivity":
                parent_treeview.insert("", tk.END, values=[isotopes, round(xi_mean, 4), round(xi_std, 4)])
            else:
                #parent_treeview.insert("", tk.END, values=[isotopes, 0.0, 0.0, round(xi_mean, 4), round(xi_std, 4)])
                self.entries_xi_mu[i][2].delete(0, tk.END)
                self.entries_xi_mu[i][2].insert(0, round(xi_mean, 4))
                self.entries_xi_std[i][2].delete(0, tk.END)
                self.entries_xi_std[i][2].insert(0, round(xi_std, 4))
                #
                i += 1
        #
        if key_plot == True:
            X = self.isotopes[:, 0]
            xi_results = np.array(xi_results, dtype=object)
            Y = np.array(xi_results)[:, 1]
            #
            self.make_bar_plot(X=X, Y=Y, parent=parent_plot)
            self.make_histo_plot(X=X, Y=Y, parent=parent_plot)
    #
    def option_changed_srm(self, op):
        #
        for i in self.srm_treeview.get_children():
            self.srm_treeview.delete(i)
        if len(self.srm_values) > 0:
            self.srm_values.clear()
        #
        path = os.getcwd()
        parent = os.path.dirname(path)
        if self.var_srm.get() == "NIST 606":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_606.csv"))
        elif self.var_srm.get() == "NIST 610":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_610.csv"))
        elif self.var_srm.get() == "NIST 610 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_610_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 610 (Spandler)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_610_Spandler.csv"))
        elif self.var_srm.get() == "NIST 611":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_611.csv"))
        elif self.var_srm.get() == "NIST 611 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_611_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 612":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_612.csv"))
        elif self.var_srm.get() == "NIST 612 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_612_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 613":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_613.csv"))
        elif self.var_srm.get() == "NIST 613 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_613_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 614":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_614.csv"))
        elif self.var_srm.get() == "NIST 614 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_614_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 615":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_615.csv"))
        elif self.var_srm.get() == "NIST 615 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_615_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 616":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_616.csv"))
        elif self.var_srm.get() == "NIST 616 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_616_GeoReM.csv"))
        elif self.var_srm.get() == "NIST 617":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_617.csv"))
        elif self.var_srm.get() == "NIST 617 (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/NIST_617_GeoReM.csv"))
        elif self.var_srm.get() == "USGS BCR-2G (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/USGS_BCR2G_GeoReM.csv"))
        elif self.var_srm.get() == "USGS GSD-1G (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/USGS_GSD1G_GeoReM.csv"))
        elif self.var_srm.get() == "USGS GSE-1G (GeoReM)":
            data_srm = data.general().importSRM(filename=parent+str("/lib/USGS_GSE1G_GeoReM.csv"))

        self.isotopes_measured = np.array(data_srm)[:, 0]
        for element in self.pse_list:
            if element in self.isotopes_measured:
                for name, conc in data_srm:
                    if name == element:
                        self.srm_treeview.insert("", tk.END, values=[element, conc])
                        self.srm_values.append(float(conc))
            else:
                self.srm_treeview.insert("", tk.END, values=[element, 0.0])
                self.srm_values.append(0.0)
    #
    def load_srm_from_csv(self, parent):
        #
        for i in self.srm_treeview.get_children():
            self.srm_treeview.delete(i)
        if len(self.srm_values) > 0:
            self.srm_values.clear()
        #
        filename = fd.askopenfilenames(parent=parent, filetypes=(("csv files", "*.csv"), ("all files", "*.*")),
                                       initialdir=os.getcwd())
        data_srm = data.general().importSRM(filename=filename[0])
        #
        self.isotopes_measured = np.array(data_srm)[:, 0]
        for element in self.pse_list:
            if element in self.isotopes_measured:
                for name, conc in data_srm:
                    if name == element:
                        self.srm_treeview.insert("", tk.END, values=[element, conc])
                        self.srm_values.append(float(conc))
            else:
                self.srm_treeview.insert("", tk.END, values=[element, 0.0])
                self.srm_values.append(0.0)
    #
    def load_conc_from_csv(self, parent, parent_tv, values_list):
        #
        for i in parent_tv.get_children():
            parent_tv.delete(i)
        if len(values_list) > 0:
            values_list.clear()
        #
        filename = fd.askopenfilenames(parent=parent, filetypes=(("csv files", "*.csv"), ("all files", "*.*")),
                                       initialdir=os.getcwd())
        data_conc = data.general().importSRM(filename=filename[0])
        #
        self.isotopes_measured = np.array(data_conc)[:, 0]
        for element in self.pse_list:
            if element in self.isotopes_measured:
                for name, conc in data_conc:
                    if name == element:
                        parent_tv.insert("", tk.END, values=[element, conc])
                        values_list.append(float(conc))
            else:
                parent_tv.insert("", tk.END, values=[element, 0.0])
                values_list.append(0.0)
    #
    def make_bar_plot(self, X, Y, parent):
        #
        try:
            self.fig.clf()
            self.ax.cla()
            self.canvas.get_tk_widget().pack_forget()
        except AttributeError:
            pass

        try:
            if self.canvas:
                self.canvas.destroy()
        except AttributeError:
            pass
        #
        self.canvas = None
        self.fig = Figure(figsize=(10, 5), facecolor=self.color_bg)
        self.ax = self.fig.add_subplot()
        #
        self.ax.axhline(1.0, color="tomato", linestyle="dashed")
        self.ax.bar(X, Y)
        self.ax.set_xticklabels(X, rotation=90)
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_axisbelow(True)
        self.ax.set_ylabel("Sensitivity $\\xi$", labelpad=0.5)

        self.plotting_area = tk.Frame(parent, bg=self.color_bg)
        self.plotting_area.grid(row=0, column=4, rowspan=14, columnspan=8, sticky="nesw")
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.plotting_area)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
    #
    def make_histo_plot(self, X, Y, parent):
        #
        try:
            self.fig_histo.clf()
            self.ax_histo.cla()
            self.canvas_histo.get_tk_widget().pack_forget()
        except AttributeError:
            pass

        try:
            if self.canvas_histo:
                self.canvas_histo.destroy()
        except AttributeError:
            pass
        #
        self.canvas_histo = None
        self.fig_histo = Figure(figsize=(10, 5), facecolor=self.color_bg)
        self.ax_histo = self.fig_histo.add_subplot()
        #
        self.ax_histo.axhline(1.0, color="tomato", linestyle="dashed")
        self.ax_histo.hist(Y, bins=50)
        self.ax_histo.grid(True)
        self.ax_histo.set_yscale("log")
        self.ax_histo.set_axisbelow(True)
        self.ax_histo.set_ylabel("Sensitivity $\\xi$", labelpad=0.5)

        self.plotting_area_histo = tk.Frame(parent, bg=self.color_bg)
        self.plotting_area_histo.grid(row=14, column=4, rowspan=12, columnspan=8, sticky="nesw")
        self.canvas_histo = FigureCanvasTkAgg(self.fig_histo, master=self.plotting_area_histo)
        self.canvas_histo.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
    #
    def calculate_concentrations(self):
        # GEOMETRY
        width = int(4*85 + 15 + 9*90 + 140 + 2*150 + 15)
        height = int(2*20 + (self.n_isotopes+3)*30 + 15)
        self.ci_window = tk.Toplevel(self.parent)
        self.ci_window.geometry(str(width)+"x"+str(height))
        self.ci_window.title("Sample Concentrations")
        #
        self.ci_values = []
        #
        self.pse_list = PSE().get_element_names()
        #
        for y in range(int(2 + self.n_isotopes + 4)):
            tk.Grid.rowconfigure(self.ci_window, y, weight=1)
        for x in range(18):
            tk.Grid.columnconfigure(self.ci_window, x, weight=1)
        #
        # Rows
        for i in range(0, 2):
            self.ci_window.grid_rowconfigure(i, minsize=20)
        for i in range(2, int(2+self.n_isotopes+4)):
            self.ci_window.grid_rowconfigure(i, minsize=30)
        self.ci_window.grid_rowconfigure(int(2+self.n_isotopes+4), minsize=15)
        # Columns
        for i in range(0, 4):
            self.ci_window.grid_columnconfigure(i, minsize=85)
        self.ci_window.grid_columnconfigure(4, minsize=15)
        for i in range(5, 14):
            self.ci_window.grid_columnconfigure(i, minsize=90)
        self.ci_window.grid_columnconfigure(14, minsize=140)
        self.ci_window.grid_columnconfigure(15, minsize=150)
        self.ci_window.grid_columnconfigure(16, minsize=150)
        self.ci_window.grid_columnconfigure(17, minsize=15)
        #
        # VARIABLES
        self.conc_smpl_is = []
        self.var_is_default = tk.StringVar()
        self.var_file_std_default = tk.StringVar()
        self.var_file_smpl_default = tk.StringVar()
        self.make_variable_file_std()
        self.make_variable_file_smpl()

        isotope_names = self.isotopes[:, 0]
        self.results_concentration = [[item] for item in isotope_names]
        #
        # Labels
        lbl_03 = tk.Label(self.ci_window, text="Isotope")
        lbl_03.grid(row=0, column=5, rowspan=2, sticky="nesw")
        lbl_04_a = tk.Label(self.ci_window, text="Concentration")
        lbl_04_a.grid(row=0, column=6, columnspan=2, sticky="nesw")
        lbl_04_b = tk.Label(self.ci_window, text="\u03BC")  # Concentration Mean
        lbl_04_b.grid(row=1, column=6, sticky="nesw")
        lbl_04_c = tk.Label(self.ci_window, text="\u03C3")  # Concentration Standard Deviation
        lbl_04_c.grid(row=1, column=7, sticky="nesw")
        lbl_05_a = tk.Label(self.ci_window, text="Sensitivity")
        lbl_05_a.grid(row=0, column=8, columnspan=2, sticky="nesw")
        lbl_05_b = tk.Label(self.ci_window, text="\u03BC")  # Sensitivity Mean
        lbl_05_b.grid(row=1, column=8, sticky="nesw")
        lbl_05_c = tk.Label(self.ci_window, text="\u03C3")  # Sensitivity Standard Deviation
        lbl_05_c.grid(row=1, column=9, sticky="nesw")
        lbl_06 = tk.Label(self.ci_window, text="Standard Reference Material")
        lbl_06.grid(row=11, column=0, columnspan=3, sticky="nesw")
        lbl_07 = tk.Label(self.ci_window, text="Sample Concentration IS")
        lbl_07.grid(row=19, column=0, columnspan=3, sticky="nesw")
        lbl_08_a = tk.Label(self.ci_window, text="Internal")
        lbl_08_a.grid(row=0, column=14, sticky="nesw")
        lbl_08_b = tk.Label(self.ci_window, text="Standard")
        lbl_08_b.grid(row=1, column=14, sticky="nesw")
        lbl_09_a = tk.Label(self.ci_window, text="Input file")
        lbl_09_a.grid(row=0, column=15, columnspan=2, sticky="nesw")
        lbl_09_b = tk.Label(self.ci_window, text="Standard")
        lbl_09_b.grid(row=1, column=15, sticky="nesw")
        lbl_09_c = tk.Label(self.ci_window, text="Sample")
        lbl_09_c.grid(row=1, column=16, sticky="nesw")

        lbl_10 = tk.Label(self.ci_window, text="General Settings")
        lbl_10.grid(row=0, column=0, rowspan=2, columnspan=4, sticky="nesw")
        lbl_bg = tk.Label(self.ci_window, text="Integration Window (BG)")
        lbl_bg.grid(row=2, column=2, columnspan=2, sticky="nesw")
        lbl_sig = tk.Label(self.ci_window, text="Integration Window (SIG)")
        lbl_sig.grid(row=6, column=2, columnspan=2, sticky="nesw")
        lbl_11_a = tk.Label(self.ci_window, text="Relative Sensitivity Factor")
        lbl_11_a.grid(row=0, column=10, columnspan=2, sticky="nesw")
        lbl_11_b = tk.Label(self.ci_window, text="\u03BC")  # RSF Mean
        lbl_11_b.grid(row=1, column=10, sticky="nesw")
        lbl_11_c = tk.Label(self.ci_window, text="\u03C3")  # RSF Standard Deviation
        lbl_11_c.grid(row=1, column=11, sticky="nesw")
        lbl_12_a = tk.Label(self.ci_window, text="Limit of Detection")
        lbl_12_a.grid(row=0, column=12, columnspan=2, sticky="nesw")
        lbl_12_b = tk.Label(self.ci_window, text="\u03BC")  # RSF Mean
        lbl_12_b.grid(row=1, column=12, sticky="nesw")
        lbl_12_c = tk.Label(self.ci_window, text="\u03C3")  # RSF Standard Deviation
        lbl_12_c.grid(row=1, column=13, sticky="nesw")
        #
        # Treeviews
        self.srm_treeview = self.make_treeview_srm(parent=self.ci_window, row_id=12, n_rows=7)
        self.conc_is_treeview = self.make_treeview_srm(parent=self.ci_window, row_id=20, n_rows=7)
        #
        # Options menus
        self.make_option_srm(parent=self.ci_window, row_id=2, column_id=0, n_columns=2) # Select SRM data
        self.make_option_is(parent=self.ci_window, row_id=int(2+self.n_isotopes), column_id=14)  # Select default internal standard
        self.make_option_file_std(parent=self.ci_window, row_id=int(2+self.n_isotopes), column_id=15)   # Select default standard file
        self.make_option_file_smpl(parent=self.ci_window, row_id=int(2+self.n_isotopes), column_id=16)   # Select default sample file
        self.make_option_iw_bg(parent=self.ci_window, row_id=3, column_id=2, n_columns=2)  # Select time integration window (background)
        self.make_option_iw(parent=self.ci_window, row_id=7, column_id=2, n_columns=2)  # Select time integration window
        #
        # Buttons
        self.make_button_srm(parent=self.ci_window, row_id=3, column_id=0, n_columns=2)
        self.make_button_smpl_conc(parent=self.ci_window, row_id=4, column_id=0, n_columns=2)
        self.make_button_advstat(parent=self.ci_window, row_id=5, column_id=0, n_columns=2)
        self.make_button_export(parent=self.ci_window, row_id=6, column_id=0, n_rows=2, n_columns=2)
        #
        # Entries Time Interval
        self.make_time_window_bg_entries(parent=self.ci_window, row_id=4, column_id=2)
        self.make_time_window_sig_entries(parent=self.ci_window, row_id=8, column_id=2)
        #
        self.var_file_indiv = []
        self.make_isotope_column(parent=self.ci_window, row_id=2, column_id=5)
        self.make_concentration_entries(parent=self.ci_window, row_id=2, column_id=6)
        self.make_sensitivity_entries(parent=self.ci_window, row_id=2, column_id=8)
        self.make_rsf_entries(parent=self.ci_window, row_id=2, column_id=10)
        self.make_lod_entries(parent=self.ci_window, row_id=2, column_id=12)
        self.make_option_is_column(parent=self.ci_window, row_id=2, column_id=14)
        self.make_option_file_column_std(parent=self.ci_window, row_id=2, column_id=15)
        self.make_option_file_column_smpl(parent=self.ci_window, row_id=2, column_id=16)
    #
    def make_option_srm(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        #
        self.srm_values = []
        option_list_srm = np.array([["NIST 606"], ["NIST 610"], ["NIST 610 (GeoReM)"], ["NIST 610 (Spandler)"],
                                     ["NIST 611"], ["NIST 611 (GeoReM)"], ["NIST 612"], ["NIST 612 (GeoReM)"],
                                     ["NIST 613"], ["NIST 613 (GeoReM)"], ["NIST 614"], ["NIST 614 (GeoReM)"],
                                     ["NIST 615"], ["NIST 615 (GeoReM)"], ["NIST 616"], ["NIST 616 (GeoReM)"],
                                     ["NIST 617"], ["NIST 617 (GeoReM)"], ["USGS BCR-2G (GeoReM)"],
                                     ["USGS GSD-1G (GeoReM)"], ["USGS GSE-1G (GeoReM)"]])[:, 0]
        self.var_srm = tk.StringVar()
        self.var_srm.set("Select SRM")
        opt_menu_srm = tk.OptionMenu(parent, self.var_srm, *option_list_srm, command=self.option_changed_srm)
        opt_menu_srm.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_option_is(self, parent, row_id, column_id, n_rows=1, n_columns=1, default_is=True, var_is=None):
        option_list_is = self.isotopes[:, 0]
        self.var_is = tk.StringVar()
        if default_is == True:
            self.var_is_default.set("Select Isotope")
            opt_menu_is = tk.OptionMenu(parent, self.var_is_default, *option_list_is, command=lambda op, var=self.var_is_default, is_default=True: self.option_changed_conc(var, is_default, op))
            opt_menu_is.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
        else:
            var_is.set("Select Isotope")
            opt_menu_is = tk.OptionMenu(parent, var_is, *option_list_is, command=lambda op, var=var_is: self.option_changed_conc(var, op))
            self.entr_is_indiv.append(opt_menu_is)
            opt_menu_is.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_option_iw(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        option_list_iw = []
        if len(self.times_def) == 0:
            option_list_iw.append(["No Time Intervals"])
        else:
            for i in range(len(self.times_def)):
                option_list_iw.append([str(self.times_def[i][0])+" - "+str(self.times_def[i][1])])
        option_list_iw = np.array(option_list_iw)[:, 0]
        self.var_iw_sig = tk.StringVar()
        self.var_iw_sig.set("Select Time Interval")
        opt_menu_iw = tk.OptionMenu(parent, self.var_iw_sig, *option_list_iw,
                                    command=lambda op, parent_plot=parent, key_plot=False, mode_xi=False:
                                    self.selected_integration_window(parent_plot, key_plot, mode_xi, op))
        opt_menu_iw.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_option_iw_bg(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        option_list_iw_bg = []
        if len(self.times_def_bg) == 0:
            option_list_iw_bg.append(["No Time Intervals"])
        else:
            for i in range(len(self.times_def_bg)):
                option_list_iw_bg.append([str(self.times_def_bg[i][0])+" - "+str(self.times_def_bg[i][1])])
        option_list_iw_bg = np.array(option_list_iw_bg)[:, 0]
        self.var_iw_bg = tk.StringVar()
        self.var_iw_bg.set("Select Time Interval")
        opt_menu_iw_bg = tk.OptionMenu(parent, self.var_iw_bg, *option_list_iw_bg,
                                    command=lambda op, parent_plot=parent, key_plot=False, mode_xi=False, mode="BG":
                                    self.selected_integration_window(parent_plot, key_plot, mode_xi, mode, op))
        opt_menu_iw_bg.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_button_srm(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        btn_srm = tk.Button(parent, text="Load SRM", fg=self.color_fg, bg=self.color_bg,
                           command=lambda parent=self.ci_window: self.load_srm_from_csv(parent))
        btn_srm.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_button_advstat(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        btn = tk.Button(parent, text="Advanced Statistics", fg=self.color_fg, bg=self.color_bg)
        btn.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_button_export(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        btn = tk.Button(parent, text="Export Results", fg=self.color_fg, bg=self.color_bg, command=self.export_results_as_csv)
        btn.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_button_smpl_conc(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        btn_smpl_conc = tk.Button(parent, text="Load IS Concentration", fg=self.color_fg, bg=self.color_bg,
                           command=lambda parent=self.ci_window, parent_tv=self.conc_is_treeview,
                                          values_list=self.conc_smpl_is: self.load_conc_from_csv(parent, parent_tv,
                                                                                                 values_list))
        btn_smpl_conc.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_time_window_sig_entries(self, parent, row_id, column_id):
        #
        lbl_01 = tk.Label(parent, text="Time Start")
        lbl_01.grid(row=row_id, column=column_id, sticky="nesw")
        lbl_02 = tk.Label(parent, text="Time End")
        lbl_02.grid(row=row_id+1, column=column_id, sticky="nesw")
        #
        self.t_start_sig = tk.StringVar()
        self.t_start_sig.set(self.times.iloc[0])
        self.entr_t_start_sig = tk.Entry(parent, textvariable=self.t_start_sig)
        self.entr_t_start_sig.grid(row=row_id, column=column_id+1, sticky="nesw")
        self.entr_t_start_sig.bind("<Return>", lambda event, entr=self.entr_t_start_sig, var=self.t_start_sig,
                                                 pos="start": self.find_nearest_time_conc(entr, var, pos, event))
        self.t_end_sig = tk.StringVar()
        self.t_end_sig.set(self.times.iloc[-1])
        self.entr_t_end_sig = tk.Entry(parent, textvariable=self.t_end_sig)
        self.entr_t_end_sig.grid(row=row_id+1, column=column_id+1, sticky="nesw")
        self.entr_t_end_sig.bind("<Return>", lambda event, entr=self.entr_t_end_sig, var=self.t_end_sig,
                                                pos="end": self.find_nearest_time_conc(entr, var, pos, event))
    #
    def make_time_window_bg_entries(self, parent, row_id, column_id):
        #
        lbl_01 = tk.Label(parent, text="Time Start")
        lbl_01.grid(row=row_id, column=column_id, sticky="nesw")
        lbl_02 = tk.Label(parent, text="Time End")
        lbl_02.grid(row=row_id+1, column=column_id, sticky="nesw")
        #
        self.t_start_bg = tk.StringVar()
        self.t_start_bg.set(self.times.iloc[0])
        self.entr_t_start_bg = tk.Entry(parent, textvariable=self.t_start_bg)
        self.entr_t_start_bg.grid(row=row_id, column=column_id+1, sticky="nesw")
        self.entr_t_start_bg.bind("<Return>", lambda event, entr=self.entr_t_start_bg, var=self.t_start_bg,
                                                  pos="start", mode="BG": self.find_nearest_time_conc(entr, var, pos, mode, event))
        self.t_end_bg = tk.StringVar()
        self.t_end_bg.set(self.times.iloc[-1])
        self.entr_t_end_bg = tk.Entry(parent, textvariable=self.t_end_bg)
        self.entr_t_end_bg.grid(row=row_id+1, column=column_id+1, sticky="nesw")
        self.entr_t_end_bg.bind("<Return>", lambda event, entr=self.entr_t_end_bg, var=self.t_end_bg,
                                                pos="end", mode="BG": self.find_nearest_time_conc(entr, var, pos, mode, event))
    #
    def make_treeview_conc(self, parent, row_id=0, column_id=5, n_rows=20, n_columns=5):
        # General Settings
        ttk.Style().configure("Treeview", background=self.color_bg, foreground="black",
                              fieldbackground=self.color_bg)
        my_font = font.Font(size="11", weight="normal")
        style = ttk.Style()
        style.configure("Treeview.Heading", font=my_font, background="gray", pressed_color="gray",
                        highlight_color="gray", foreground=self.color_fg)
        style.map("Treeview.Heading", background = [("pressed", "!focus", "gray"),
                                                    ("active", "gray"), ("disabled", self.color_bg)])
        #
        columns = ("#1", "#2", "#3", "#4", "#5")
        treeview = ttk.Treeview(parent, columns=columns, show="headings")
        treeview.heading("#1", text="Isotope")
        treeview.column("#1", minwidth=0, width=70, stretch=tk.NO)
        treeview.heading("#2", text="Concentration")
        treeview.column("#2", minwidth=0, width=130, stretch=tk.NO)
        treeview.heading("#3", text="Error")
        treeview.column("#3", minwidth=0, width=80, stretch=tk.NO)
        treeview.heading("#4", text="Sensitivity")
        treeview.column("#4", minwidth=0, width=90, stretch=tk.NO)
        treeview.heading("#5", text="Error")
        treeview.column("#5", minwidth=0, width=80, stretch=tk.NO)
        treeview.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
        #
        return treeview
    #
    def option_changed_conc(self, var, op, is_default=False):
        if is_default == True:
            for i in range(self.n_isotopes):
                self.var_is_indiv[i].set(var.get())
        else:
            for i in range(self.n_isotopes):
                self.var_is_indiv[i].set(var.get())
        #
        for i in range(self.n_isotopes):
            self.calculate_xi_2(var_file=self.var_file_std_default.get(), var_i=self.isotopes[:, 0][i],
                                var_is=self.var_is_default.get(), index=i)
            self.calculate_ci(var_file_std=self.var_file_std_default.get(),
                              var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                              var_i=self.isotopes[:, 0][i], index=i)
        for i in range(self.n_isotopes):
            self.calculate_rsf(var_file_std=self.var_file_std_default.get(),
                               var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                               var_i=self.isotopes[:, 0][i], index=i)
            self.calculate_lod(var_file_std=self.var_file_std_default.get(),
                               var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                               var_i=self.isotopes[:, 0][i], index=i)
    #
    def find_nearest_time_conc(self, entr, var, pos, event, mode="SIG", t_iw=None):
        #
        if t_iw == None:
            t_nearest = min(self.times, key=lambda x: abs(x - float(var.get())))
            entr.delete(0, tk.END)
            entr.insert(0, t_nearest)
            t_id = self.times[self.times == t_nearest].index[0]
            #
            if pos == "start":
                if mode == "SIG":
                    self.indices_sig[0] = t_id
                else:
                    self.indices_bg[0] = t_id
            else:
                if mode == "SIG":
                    self.indices_sig[1] = t_id
                else:
                    self.indices_bg[1] = t_id
        else:
            t_id_start = self.times[self.times == t_iw[0]].index[0]
            t_id_end = self.times[self.times == t_iw[1]].index[0]
            if mode == "SIG":
                self.indices_sig[0] = t_id_start
                self.indices_sig[1] = t_id_end
                #
                self.entr_t_start_sig.delete(0, tk.END)
                self.entr_t_start_sig.insert(0, t_iw[0])
                self.entr_t_end_sig.delete(0, tk.END)
                self.entr_t_end_sig.insert(0, t_iw[1])
            else:
                self.indices_bg[0] = t_id_start
                self.indices_bg[1] = t_id_end
                #
                self.entr_t_start_bg.delete(0, tk.END)
                self.entr_t_start_bg.insert(0, t_iw[0])
                self.entr_t_end_bg.delete(0, tk.END)
                self.entr_t_end_bg.insert(0, t_iw[1])
        #
        for i in range(self.n_isotopes):
            self.calculate_xi_2(var_file=self.var_file_std_default.get(), var_i=self.isotopes[:, 0][i],
                                var_is=self.var_is_default.get(), index=i)
            self.calculate_ci(var_file_std=self.var_file_std_default.get(),
                              var_file_smpl=self.var_file_smpl_default.get(), var_is=self.var_is_default.get(),
                              var_i=self.isotopes[:, 0][i], index=i)
            self.calculate_rsf(var_file_std=self.var_file_std_default.get(), var_file_smpl=self.var_file_smpl_default.get(),
                           var_is=self.var_is_default.get(), var_i=self.isotopes[:, 0][i], index=i)
            self.calculate_lod(var_file_std=self.var_file_std_default.get(), var_file_smpl=self.var_file_smpl_default.get(),
                           var_is=self.var_is_default.get(), var_i=self.isotopes[:, 0][i], index=i)
    #
    def make_isotope_column(self, parent, row_id, column_id):
        i = 0
        for isotope in self.isotopes[:, 0]:
            lbl_isotope = tk.Label(parent, text=isotope)
            lbl_isotope.grid(row=row_id+i, column=column_id, sticky="nesw")
            i += 1
    #
    def make_concentration_entries(self, parent, row_id, column_id):
        i = 0
        self.variables_conc_mu = []
        self.entries_conc_mu = []
        self.variables_conc_std = []
        self.entries_conc_std = []
        for isotope in self.isotopes[:, 0]:
            self.variables_conc_mu.append([isotope, tk.StringVar()])
            self.variables_conc_mu[i][1].set(0.0)
            self.entries_conc_mu.append([isotope, self.variables_conc_mu[i][1],
                                      tk.Entry(parent, textvariable=self.variables_conc_mu[i][1])])
            self.entries_conc_mu[i][2].grid(row=row_id+i, column=column_id, sticky="nesw")
            #
            self.variables_conc_std.append([isotope, tk.StringVar()])
            self.variables_conc_std[i][1].set(0.0)
            self.entries_conc_std.append([isotope, self.variables_conc_std[i][1],
                                      tk.Entry(parent, textvariable=self.variables_conc_std[i][1])])
            self.entries_conc_std[i][2].grid(row=row_id+i, column=column_id+1, sticky="nesw")
            #
            i += 1
    #
    def make_sensitivity_entries(self, parent, row_id, column_id):
        i = 0
        self.variables_xi_mu = []
        self.entries_xi_mu = []
        self.variables_xi_std = []
        self.entries_xi_std = []
        for isotope in self.isotopes[:, 0]:
            self.variables_xi_mu.append([isotope, tk.StringVar()])
            self.variables_xi_mu[i][1].set(0.0)
            self.entries_xi_mu.append([isotope, self.variables_xi_mu[i][1],
                                      tk.Entry(parent, textvariable=self.variables_xi_mu[i][1])])
            self.entries_xi_mu[i][2].grid(row=row_id+i, column=column_id, sticky="nesw")
            #
            self.variables_xi_std.append([isotope, tk.StringVar()])
            self.variables_xi_std[i][1].set(0.0)
            self.entries_xi_std.append([isotope, self.variables_xi_std[i][1],
                                      tk.Entry(parent, textvariable=self.variables_xi_std[i][1])])
            self.entries_xi_std[i][2].grid(row=row_id+i, column=column_id+1, sticky="nesw")
            #
            i += 1
    #
    def make_rsf_entries(self, parent, row_id, column_id):
        i = 0
        self.variables_rsf_mu = []
        self.entries_rsf_mu = []
        self.variables_rsf_std = []
        self.entries_rsf_std = []
        for isotope in self.isotopes[:, 0]:
            self.variables_rsf_mu.append([isotope, tk.StringVar()])
            self.variables_rsf_mu[i][1].set(0.0)
            self.entries_rsf_mu.append([isotope, self.variables_rsf_mu[i][1],
                                      tk.Entry(parent, textvariable=self.variables_rsf_mu[i][1])])
            self.entries_rsf_mu[i][2].grid(row=row_id+i, column=column_id, sticky="nesw")
            #
            self.variables_rsf_std.append([isotope, tk.StringVar()])
            self.variables_rsf_std[i][1].set(0.0)
            self.entries_rsf_std.append([isotope, self.variables_rsf_std[i][1],
                                      tk.Entry(parent, textvariable=self.variables_rsf_std[i][1])])
            self.entries_rsf_std[i][2].grid(row=row_id+i, column=column_id+1, sticky="nesw")
            #
            i += 1
    #
    def make_lod_entries(self, parent, row_id, column_id):
        i = 0
        self.variables_lod_mu = []
        self.entries_lod_mu = []
        self.variables_lod_std = []
        self.entries_lod_std = []
        for isotope in self.isotopes[:, 0]:
            self.variables_lod_mu.append([isotope, tk.StringVar()])
            self.variables_lod_mu[i][1].set(0.0)
            self.entries_lod_mu.append([isotope, self.variables_lod_mu[i][1],
                                      tk.Entry(parent, textvariable=self.variables_lod_mu[i][1])])
            self.entries_lod_mu[i][2].grid(row=row_id+i, column=column_id, sticky="nesw")
            #
            self.variables_lod_std.append([isotope, tk.StringVar()])
            self.variables_lod_std[i][1].set(0.0)
            self.entries_lod_std.append([isotope, self.variables_lod_std[i][1],
                                      tk.Entry(parent, textvariable=self.variables_lod_std[i][1])])
            self.entries_lod_std[i][2].grid(row=row_id+i, column=column_id+1, sticky="nesw")
            #
            i += 1
    #
    def make_option_is_column(self, parent, row_id=2, column_id=10):
        self.var_is_indiv = []
        option_list_is = self.isotopes[:, 0]
        for i in range(self.n_isotopes):
            self.var_is_indiv.append(tk.StringVar())
            self.var_is_indiv[i].set("Select isotope")
            opt_menu_is = tk.OptionMenu(parent, self.var_is_indiv[i], *option_list_is,
                                        command=lambda op, var_file_std=self.var_file_std_indiv[i],
                                                       var_file_smpl=self.var_file_smpl_indiv[i], index=i,
                                                       var_is=self.var_is_indiv[i]: self.option_changed_is(var_file_std,
                                                                                                           var_file_smpl,
                                                                                                           index, var_is,
                                                                                                           op))
            opt_menu_is.grid(row=row_id+i, column=column_id, rowspan=1, columnspan=1, sticky="nesw")
    #
    def make_variable_file_std(self):
        self.var_file_std_indiv = []
        file_helper = ["All Standards"]
        for file in self.files_std:
            parts = file.split("/")
            file_helper.append(parts[-1])
        parts_actual = self.file_actual.split("/")
        self.option_list_files_std = np.array(file_helper)
        for i in range(self.n_isotopes):
            self.var_file_std_indiv.append(tk.StringVar())
            if self.file_actual in self.files_std:
                self.var_file_std_indiv[i].set(parts_actual[-1])
            else:
                self.var_file_std_indiv[i].set(self.option_list_files_std[1])
    #
    def make_variable_file_smpl(self):
        self.var_file_smpl_indiv = []
        file_helper = ["All Samples"]
        for file in self.files_smpl:
            parts = file.split("/")
            file_helper.append(parts[-1])
        parts_actual = self.file_actual.split("/")
        self.option_list_files_smpl = np.array(file_helper)
        for i in range(self.n_isotopes):
            self.var_file_smpl_indiv.append(tk.StringVar())
            if self.file_actual in self.files_smpl:
                self.var_file_smpl_indiv[i].set(parts_actual[-1])
            else:
                self.var_file_smpl_indiv[i].set(self.option_list_files_smpl[1])
    #
    def make_option_file_std(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        parts_actual = self.file_actual.split("/")
        if self.file_actual in self.files_std:
                self.var_file_std_default.set(parts_actual[-1])
        else:
            self.var_file_std_default.set(self.option_list_files_std[1])
        opt_menu_file_std = tk.OptionMenu(parent, self.var_file_std_default, *self.option_list_files_std,
                                          command=lambda op, var_default=self.var_file_std_default,
                                                         var_indiv=self.var_file_std_indiv, key="standard": self.option_changed_file_default(var_default, var_indiv, key, op))
        opt_menu_file_std.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def make_option_file_smpl(self, parent, row_id, column_id, n_rows=1, n_columns=1):
        parts_actual = self.file_actual.split("/")
        if self.file_actual in self.files_smpl:
                self.var_file_smpl_default.set(parts_actual[-1])
        else:
            self.var_file_smpl_default.set(self.option_list_files_smpl[1])
        opt_menu_file_smpl = tk.OptionMenu(parent, self.var_file_smpl_default, *self.option_list_files_smpl,
                                          command=lambda op, var_default=self.var_file_smpl_default,
                                                         var_indiv=self.var_file_smpl_indiv, key="sample": self.option_changed_file_default(var_default, var_indiv, key, op))
        opt_menu_file_smpl.grid(row=row_id, column=column_id, rowspan=n_rows, columnspan=n_columns, sticky="nesw")
    #
    def option_changed_file_default(self, var_default, var_indiv, key, op):
        if key == "standard":
            for i in range(self.n_isotopes):
                var_indiv[i].set(var_default.get())
                self.calculate_xi_2(var_file=var_indiv[i].get(), var_i=self.isotopes[:, 0][i], var_is=self.var_is_indiv[i].get(), index=i)
                self.calculate_ci(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(), var_is=self.var_is_indiv[i].get(),
                                  var_i=self.isotopes[:, 0][i], index=i)
                self.calculate_rsf(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(),
                           var_is=self.var_is_indiv[i].get(), var_i=self.isotopes[:, 0][i], index=i)
                self.calculate_lod(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(),
                           var_is=self.var_is_indiv[i].get(), var_i=self.isotopes[:, 0][i], index=i)
        elif key == "sample":
            for i in range(self.n_isotopes):
                var_indiv[i].set(var_default.get())
                self.calculate_ci(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(), var_is=self.var_is_indiv[i].get(),
                                      var_i=self.isotopes[:, 0][i], index=i)
                self.calculate_rsf(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(),
                           var_is=self.var_is_indiv[i].get(), var_i=self.isotopes[:, 0][i], index=i)
                self.calculate_lod(var_file_std=self.var_file_std_indiv[i].get(), var_file_smpl=self.var_file_smpl_indiv[i].get(),
                           var_is=self.var_is_indiv[i].get(), var_i=self.isotopes[:, 0][i], index=i)
    #
    def make_option_file_column_std(self, parent, row_id=2, column_id=11):
        for i in range(self.n_isotopes):
            opt_menu_file = tk.OptionMenu(parent, self.var_file_std_indiv[i], *self.option_list_files_std,
                                          command=lambda op, var_file_std=self.var_file_std_indiv[i],
                                                         var_file_smpl=self.var_file_smpl_indiv[i], index=i,
                                                         var_is=self.var_is_indiv[i], key="standard": self.option_changed_file(var_file_std, var_file_smpl,
                                                                                                               index,
                                                                                                               var_is, key,
                                                                                                               op))
            opt_menu_file.grid(row=row_id+i, column=column_id, rowspan=1, columnspan=1, sticky="nesw")
    #
    def make_option_file_column_smpl(self, parent, row_id=2, column_id=12):
        for i in range(self.n_isotopes):
            opt_menu_file = tk.OptionMenu(parent, self.var_file_smpl_indiv[i], *self.option_list_files_smpl,
                                          command=lambda op, var_file_std=self.var_file_std_indiv[i],
                                                         var_file_smpl=self.var_file_smpl_indiv[i], index=i,
                                                         var_is=self.var_is_indiv[i], key="sample": self.option_changed_file(var_file_std, var_file_smpl,
                                                                                                               index,
                                                                                                               var_is, key,
                                                                                                               op))
            opt_menu_file.grid(row=row_id+i, column=column_id, rowspan=1, columnspan=1, sticky="nesw")
    #
    def calculate_xi_2(self, var_file, var_i, var_is, index):
        #
        key_i = re.search("(\D+)(\d+)", var_i)
        key_is = re.search("(\D+)(\d+)", var_is)
        index_i = self.pse_list.index(key_i.group(1))
        index_is = self.pse_list.index(key_is.group(1))
        #
        concentration_i = self.srm_values[index_i]
        concentration_is = self.srm_values[index_is]
        #
        if var_file not in ["All Standards", "All Samples"]:
            for item in self.files_std:
                    if var_file in item:
                        var_file = item
            #
            dataset_std = data.Data(filename=var_file)
            df_std = dataset_std.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
            #
            intensity_i = df_std[var_i][self.indices_sig[0]:self.indices_sig[1]+1].mean()
            intensity_is = df_std[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
            #
            if concentration_i == 0:
                self.entries_xi_mu[index][2].delete(0, tk.END)
                self.entries_xi_mu[index][2].insert(0, round(0.0, 8))
                self.entries_xi_std[index][2].delete(0, tk.END)
                self.entries_xi_std[index][2].insert(0, round(0.0, 8))
            else:
                xi = intensity_i/intensity_is * concentration_is/concentration_i
                self.entries_xi_mu[index][2].delete(0, tk.END)
                self.entries_xi_mu[index][2].insert(0, round(xi.mean(), 8))
                self.entries_xi_std[index][2].delete(0, tk.END)
                self.entries_xi_std[index][2].insert(0, round(0.0, 8))
        else:
            files_helper = []
            results_helper = [[], []]
            if var_file == "All Standards":
                var_file = self.option_list_files_std[1:]
                for item_1 in self.files_std:
                    for item_2 in var_file:
                        if item_2 in item_1:
                            files_helper.append(item_1)
            elif var_file == "All Samples":
                var_file = self.option_list_files_smpl[1:]
                for item_1 in self.files_smpl:
                    for item_2 in var_file:
                        if item_2 in item_1:
                            files_helper.append(item_1)
            #
            for item in files_helper:
                dataset_std_j = data.Data(filename=item)
                df_std_j = dataset_std_j.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                #
                intensity_i_j = df_std_j[var_i][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                intensity_is_j = df_std_j[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                #
                if concentration_i > 0:
                    xi_j = intensity_i_j/intensity_is_j * concentration_is/concentration_i
                    results_helper[0].append(xi_j)
                    results_helper[1].append(xi_j)
                #
            if concentration_i == 0:
                self.entries_xi_mu[index][2].delete(0, tk.END)
                self.entries_xi_mu[index][2].insert(0, round(0.0, 8))
                self.entries_xi_std[index][2].delete(0, tk.END)
                self.entries_xi_std[index][2].insert(0, round(0.0, 8))
            else:
                self.entries_xi_mu[index][2].delete(0, tk.END)
                self.entries_xi_mu[index][2].insert(0, round(np.mean(results_helper[0]), 8))
                if len(files_helper) == 1:
                    self.entries_xi_std[index][2].delete(0, tk.END)
                    self.entries_xi_std[index][2].insert(0, 0.0)
                else:
                    self.entries_xi_std[index][2].delete(0, tk.END)
                    self.entries_xi_std[index][2].insert(0, round(np.std(results_helper[1], ddof=1), 8))
    #
    def option_changed_file(self, var_file_std, var_file_smpl, index, var_is, key, op):
        if key == "standard":
            self.calculate_xi_2(var_file=var_file_std.get(), var_i=self.isotopes[:, 0][index], var_is=var_is.get(), index=index)
            self.calculate_ci(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                              var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
            self.calculate_rsf(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
            self.calculate_lod(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
        elif key == "sample":
            self.calculate_ci(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                              var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
            self.calculate_rsf(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
            self.calculate_lod(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
    #
    def option_changed_is(self, var_file_std, var_file_smpl, index, var_is, op):
        self.calculate_xi_2(var_file=var_file_std.get(), var_i=self.isotopes[:, 0][index], var_is=var_is.get(),
                            index=index)
        self.calculate_ci(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                          var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
        self.calculate_rsf(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
        self.calculate_lod(var_file_std=var_file_std.get(), var_file_smpl=var_file_smpl.get(),
                           var_is=var_is.get(), var_i=self.isotopes[:, 0][index], index=index)
    #
    def calculate_ci(self, var_file_std, var_file_smpl, var_is, var_i, index):
        files_helper_std = []
        files_helper_smpl = []
        #
        results_helper = []
        if var_file_smpl == "All Samples":
            if var_file_smpl == "All Samples":
                var_file_smpl = self.option_list_files_smpl[1:]
                for item_1 in self.files_smpl:
                    for item_2 in var_file_smpl:
                        if item_2 in item_1:
                            files_helper_smpl.append(item_1)
        else:
            for item in self.files_smpl:
                if var_file_smpl in item:
                    var_file_smpl = item
                    files_helper_smpl.append(item)
        #
        if var_file_std == "All Standards":
            if var_file_std == "All Standards":
                var_file_std = self.option_list_files_std[1:]
                for item_1 in self.files_std:
                    for item_2 in var_file_std:
                        if item_2 in item_1:
                            files_helper_std.append(item_1)
        else:
            for item in self.files_std:
                if var_file_std in item:
                    var_file_std = item
                    files_helper_std.append(item)
        #
        if len(self.results_concentration[index]) > 1:
            self.results_concentration[index] = self.results_concentration[index][:1]
        self.results_concentration[index].append(var_is)
        #
        key_i = re.search("(\D+)(\d+)", var_i)
        key_is = re.search("(\D+)(\d+)", var_is)
        index_i = self.pse_list.index(key_i.group(1))
        index_is = self.pse_list.index(key_is.group(1))
        #
        concentration_i = self.srm_values[index_i]
        concentration_is = self.srm_values[index_is]
        concentration_is_smpl = self.conc_smpl_is[index_is]
        #
        for item_std in files_helper_std:
            for item in files_helper_smpl:
                dataset_std_k = data.Data(filename=item_std)
                df_std_k = dataset_std_k.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                dataset_smpl_j = data.Data(filename=item)
                df_smpl_j = dataset_smpl_j.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                #
                intensity_i_k = df_std_k[var_i][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                intensity_is_k = df_std_k[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                intensity_i_j = df_smpl_j[var_i][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                intensity_is_j = df_smpl_j[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                if concentration_i > 0:
                    xi_self = (intensity_i_k/intensity_is_k) * (concentration_is/concentration_i)
                else:
                    xi_self = 0.0
                xi_j = float(self.entries_xi_mu[index][2].get())
                #
                if xi_j > 0 and xi_self > 0:
                    #ci_j = (intensity_i_j/intensity_is_j)*(concentration_is_smpl/xi_j)
                    ci_jk = (intensity_i_j/intensity_is_j)*(concentration_is_smpl/xi_j) * (intensity_i_k/intensity_is_k) * (concentration_is/concentration_i)
                    results_helper.append(ci_jk)
                else:
                    pass
        #
        if concentration_i > 0:
            conc_mu = np.mean(results_helper)
            self.entries_conc_mu[index][2].delete(0, tk.END)
            self.entries_conc_mu[index][2].insert(0, round(conc_mu, 8))
            self.results_concentration[index].append(conc_mu)
            if len(files_helper_smpl)*len(files_helper_std) == 1:
                cond_std = 0.0
                self.entries_conc_std[index][2].delete(0, tk.END)
                self.entries_conc_std[index][2].insert(0, cond_std)
                self.results_concentration[index].append(cond_std)
            else:
                conc_std = np.std(results_helper, ddof=1)
                self.entries_conc_std[index][2].delete(0, tk.END)
                self.entries_conc_std[index][2].insert(0, round(conc_std, 8))
                self.results_concentration[index].append(conc_std)
        else:
            self.results_concentration[index].append(0.0)
            self.results_concentration[index].append(0.0)
    #
    def calculate_rsf(self, var_file_std, var_file_smpl, var_is, var_i, index):
        files_helper_std = []
        files_helper_smpl = []
        results_helper = []
        #
        if var_file_smpl == "All Samples":
            if var_file_smpl == "All Samples":
                var_file_smpl = self.option_list_files_smpl[1:]
                for item_1 in self.files_smpl:
                    for item_2 in var_file_smpl:
                        if item_2 in item_1:
                            files_helper_smpl.append(item_1)
        else:
            for item in self.files_smpl:
                if var_file_smpl in item:
                    var_file_smpl = item
                    files_helper_smpl.append(item)
        #
        if var_file_std == "All Standards":
            if var_file_std == "All Standards":
                var_file_std = self.option_list_files_std[1:]
                for item_1 in self.files_std:
                    for item_2 in var_file_std:
                        if item_2 in item_1:
                            files_helper_std.append(item_1)
        else:
            for item in self.files_std:
                if var_file_std in item:
                    var_file_std = item
                    files_helper_std.append(item)
        #
        if len(files_helper_smpl) == 0 and len(files_helper_std) == 0:
            dataset_std = data.Data(filename=var_file_std)
            df_std = dataset_std.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
            dataset_smpl = data.Data(filename=var_file_smpl)
            df_smpl = dataset_smpl.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
            #
            key_is = re.search("(\D+)(\d+)", var_is)
            index_is = self.isotopes[:, 0].tolist().index(key_is.group(0))
            key_is_std = re.search("(\D+)(\d+)", var_is)
            index_is_std = self.pse_list.index(key_is_std.group(1))
            #
            intensity_is_std = df_std[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
            intensity_is_smpl = df_smpl[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
            c_is_std = self.srm_values[index_is_std]
            c_is_smpl = float(self.entries_conc_mu[index_is][2].get())
            #
            if c_is_smpl > 0:
                rsf_i = (c_is_std/c_is_smpl) * (intensity_is_smpl/intensity_is_std)
                self.entries_rsf_mu[index][2].delete(0, tk.END)
                self.entries_rsf_mu[index][2].insert(0, round(rsf_i, 8))
                self.entries_rsf_std[index][2].delete(0, tk.END)
                self.entries_rsf_std[index][2].insert(0, round(0.0, 8))
            else:
                pass
        else:
            key_is = re.search("(\D+)(\d+)", var_is)
            index_is = self.isotopes[:, 0].tolist().index(key_is.group(0))
            key_is_std = re.search("(\D+)(\d+)", var_is)
            index_is_std = self.pse_list.index(key_is_std.group(1))
            #
            c_is_std = self.srm_values[index_is_std]
            c_is_smpl = float(self.entries_conc_mu[index_is][2].get())
            #
            for item_std in files_helper_std:
                for item_smpl in files_helper_smpl:
                    dataset_std_j = data.Data(filename=item_std)
                    df_std_j = dataset_std_j.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                    dataset_smpl_j = data.Data(filename=item_smpl)
                    df_smpl_j = dataset_smpl_j.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                    #
                    intensity_is_std_j = df_std_j[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                    intensity_is_smpl_j = df_smpl_j[var_is][self.indices_sig[0]:self.indices_sig[1]+1].mean()
                    #
                    if c_is_smpl > 0:
                        rsf_j = (c_is_std/c_is_smpl) * (intensity_is_smpl_j/intensity_is_std_j)
                        results_helper.append(rsf_j)
                #
                if c_is_smpl > 0:
                    self.entries_rsf_mu[index][2].delete(0, tk.END)
                    self.entries_rsf_mu[index][2].insert(0, round(np.mean(results_helper), 8))
                    if len(results_helper) == 1:
                        self.entries_rsf_std[index][2].delete(0, tk.END)
                        self.entries_rsf_std[index][2].insert(0, 0.0)
                    else:
                        self.entries_rsf_std[index][2].delete(0, tk.END)
                        self.entries_rsf_std[index][2].insert(0, round(np.std(results_helper, ddof=1), 8))
                else:
                    pass
    #
    def calculate_lod(self, var_file_std, var_file_smpl, var_is, var_i, index):
        files_helper_std = []
        files_helper_smpl = []
        results_helper = []
        #
        if var_file_smpl == "All Samples":
            if var_file_smpl == "All Samples":
                var_file_smpl = self.option_list_files_smpl[1:]
                for item_1 in self.files_smpl:
                    for item_2 in var_file_smpl:
                        if item_2 in item_1:
                            files_helper_smpl.append(item_1)
        else:
            for item in self.files_smpl:
                if var_file_smpl in item:
                    var_file_smpl = item
                    files_helper_smpl.append(item)
        #
        if var_file_std == "All Standards":
            if var_file_std == "All Standards":
                var_file_std = self.option_list_files_std[1:]
                for item_1 in self.files_std:
                    for item_2 in var_file_std:
                        if item_2 in item_1:
                            files_helper_std.append(item_1)
        else:
            for item in self.files_std:
                if var_file_std in item:
                    var_file_std = item
                    files_helper_std.append(item)
        #
        key_i_std = re.search("(\D+)(\d+)", var_i)
        index_i_std = self.pse_list.index(key_i_std.group(1))
        #
        c_i_std = self.srm_values[index_i_std]
        #
        n_bg = len(self.times[self.indices_bg[0]:self.indices_bg[1]+1])
        n_sig = len(self.times[self.indices_sig[0]:self.indices_sig[1]+1])
        #
        rsf_i = float(self.entries_rsf_mu[index][2].get())
        #
        for item_std in files_helper_std:
            dataset_std_j = data.Data(filename=item_std)
            df_std_j = dataset_std_j.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
            #
            intensity_i_std_j = df_std_j[var_i][self.indices_sig[0]:self.indices_sig[1]+1].mean()
            sigma_i_j = df_std_j[var_i][self.indices_bg[0]:self.indices_bg[1]+1].std()
            #
            if rsf_i > 0:
                lod_j = (3*sigma_i_j)/rsf_i * np.sqrt((1/n_bg) + (1/n_sig)) * (c_i_std/intensity_i_std_j)
                results_helper.append(lod_j)
        #
        if rsf_i > 0:
            self.entries_lod_mu[index][2].delete(0, tk.END)
            self.entries_lod_mu[index][2].insert(0, round(np.mean(results_helper), 8))
            if len(files_helper_std) == 1:
                self.entries_lod_std[index][2].delete(0, tk.END)
                self.entries_lod_std[index][2].insert(0, 0.0)
            else:
                self.entries_lod_std[index][2].delete(0, tk.END)
                self.entries_lod_std[index][2].insert(0, round(np.std(results_helper, ddof=1), 8))
        else:
            pass
    #
    def export_results_as_csv(self):
        print(self.results_concentration)
       # results_csv = open("results.csv", "w")
       # results_csv.write("C_mu"+";"+"C_std")


# -*- coding: utf-8 -*-
# Author: Yiping Liu
# Description: This script calculates the average of a list of numbers.
# Version: 1.0
# Last Modified: May 7, 2023

import matplotlib
matplotlib.use('TkAgg')

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

# read the csv file
df = pd.read_csv("E:\\work\\data\\low-T\\error_analysis_s2_4.csv")

# extract the column of interest
Na = df['Na']

# calculate the mean and standard deviation
mean = Na.mean()
std_dev = Na.std(ddof=1) # use Bessel's correction for sample standard deviation

# calculate the 68% confidence interval
lower_bound, upper_bound = mean - std_dev, mean + std_dev
uncertainty = (upper_bound - lower_bound) / 2

# plot the histogram with the confidence interval highlighted
n, bins, patches = plt.hist(Na, bins=30, alpha=0.5, edgecolor='black')
bin_centers = 0.5 * (bins[:-1] + bins[1:])
for patch, left, right in zip(patches, bins[:-1], bins[1:]):
    if left <= lower_bound or right >= upper_bound:
        patch.set_fc('gray')
plt.axvspan(lower_bound, upper_bound, alpha=0.2, color='gray')
plt.title("Na Histogram with 68% Confidence Interval")
plt.xlabel("Na Values")
plt.ylabel("Frequency")

# print the results
print("Mean: {:.2f}".format(mean))
print("Standard Deviation: {:.2f}".format(std_dev))
print("68% Confidence Interval: {:.2f} ± {:.2f}".format(mean, uncertainty))

plt.gcf().canvas.required_interactive_framework = 'TkAgg'
plt.show()



#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# statistics.py
# Maximilian Beeskow
# 07.08.2021
# ----------------------
#
## MODULES
import numpy as np
import pandas as pd
from outliers import smirnov_grubbs as grubbs
import re
import matplotlib.pyplot as plt
from scipy import stats
#
## TOOLS
#
class statisticalAnalysis:
    #
    def __init__(self, data):
        self.data = data
    #
    def analyzeSensitivities(self, reference):
        # Input data
        xi = self.data
        self.reference = reference
        #
        #print("")
        #print("Statistical analysis of the sensitivity calculation (EXTERNAL STANDARD):")
        xiData = []
        for i in range(0, len(xi[0])):
            xiData.append([xi[0][i][0], [xi[0][i][2][0], xi[1][i][2][0], xi[2][i][2][0], xi[3][i][2][0]], [xi[0][i][2][1], xi[1][i][2][1], xi[2][i][2][1], xi[3][i][2][1]]])
        for i in range(0, len(xiData)):
           print(xiData[i][0], ":", "mean =", np.mean(xiData[i][1]), "std =", np.std(xiData[i][1], ddof=1), round(100*np.std(xiData[i][1], ddof=1)/np.mean(xiData[i][1]), 2), "%")
        print("")
        #
        data = []
        #
        for i in range(0, len(xiData)):
            data.append([xiData[i][0], np.mean(xiData[i][1]), np.std(xiData[i][1], ddof=1), self.reference])
        #
        return data
    #
    def analyze_xi(self, reference, isotopes):
        # Input data
        xi = self.data
        self.isotopes = isotopes
        self.reference = reference
        #
        #print("")
        #print("Statistical analysis of the sensitivity calculation (EXTERNAL STANDARD):")
        all_xi = [[self.isotopes[i], []] for i in range(len(self.isotopes))]
        for i in range(len(self.data[0])):
            for j in range(len(self.data)):
                all_xi[i][1].append(self.data[j][i])
                print(self.data[j][i])
        for i in range(len(all_xi)):
           print(all_xi[i][0], ":", "mean =", np.mean(all_xi[i][1]), "std =", np.std(all_xi[i][1], ddof=1), round(100*np.std(all_xi[i][1], ddof=1)/np.mean(all_xi[i][1]), 2), "%")
        print("")
        #
        data_raw = []
        for i in range(len(all_xi)):
            data_raw.append(pd.Series(all_xi[i][1]))
        df = pd.concat(data_raw, axis=1)
        df.columns = self.isotopes
        print(df)
        print(df.describe())
        data = []
        #
        for i in range(0, len(all_xi)):
            data.append([all_xi[i][0], np.mean(all_xi[i][1]), np.std(all_xi[i][1], ddof=1), self.reference])
        print(data)
        #
        return data, df
    #
    def analyzeConcentrations(self):
        # Input data
        inputC = self.data
        #
        list_meanC = []
        list_stdC = []
        for i in range(0, len(inputC[0])):
            list_meanC.append([inputC[0][i][0], []])
            list_stdC.append([inputC[0][i][0], []])
        for i in range(0, len(inputC)):
            for j in range(0, len(inputC[0])):
                if inputC[i][j][0] == list_meanC[j][0]:
                    list_meanC[j][1].append(inputC[i][j][2][0])
                    list_stdC[j][1].append(inputC[i][j][2][1])
        #print("")
        #print("Statistical analysis of the concentration calculation (SAMPLE):")
        #print("Number of measurements:", len(inputC))
        dataC = []
        for i in range(0, len(inputC[0])):
            dataC.append([inputC[0][i][0], [inputC[0][i][2][0], inputC[1][i][2][0]], [inputC[0][i][2][1], inputC[1][i][2][1]]])
        #for i in range(0, len(dataC)):
        #    print(dataC[i][0], ":", "mean =", round(np.mean(dataC[i][1]),2), "std =", round(np.mean(dataC[i][2]),2), round(100*np.mean(dataC[i][2])/np.mean(dataC[i][1]),2), "%")
        #print("")
        #
        data = []
        #
        for i in range(0, len(dataC)):
            data.append([dataC[i][0], np.mean(dataC[i][1]), np.std(dataC[i][1], ddof=1)])
        #
        return data

class SimpleStatistics:
    #
    def __init__(self, input_data):
        self.input_data = self.input_data
    #
    def convert_data_to_dataframe(self):
        pass


class Segmentation:
    #
    def __init__(self, data_input, isotopes):
        self.data_input = data_input
        self.isotopes = isotopes
    #
    def find_background_segments(self, isotope, threshold=200):
        indices = np.where(self.data_input > threshold)[0]
        print(isotope)
        # print("Mean(intensity) BG 1:", np.mean(self.data_input[0:indices[0]]))
        # print("Mean(intensity) BG 2:", np.mean(self.data_input[indices[-1]+1:-1]))
        limits_bg1 = [0, indices[0]]
        limits_bg2 = [indices[-1]+1, len(self.data_input)-1]
        # print(limits_bg1)
        # print(limits_bg2)
        #
        return [limits_bg1, limits_bg2]
    #
    def calculate_change(self, bg1_end, bg2_start):
        data = self.data_input[bg1_end:bg2_start].values
        change = np.around(np.array([(data[i]-data[i-1])/(data[i-1])*100 for i in range(1, len(data))]), 2)
        change = np.insert(change, 0, 0, axis=0)
        print(change)

class OutlierAnalysis:
    #
    def __init__(self, data_input):
        self.data_input = data_input
    #
    def grubbs_test_two_sided(self, alpha=0.05):
        """
        Performs a two-sided outlier test after Grubbs
        :param alpha: level of significance
        :return data_output: data array without outliers
        """
        data_output = grubbs.test(data=self.data_input, alpha=alpha)
        outliers = grubbs.max_test_outliers(data=self.data_input, alpha=.05)
        outlier_indices = grubbs.max_test_indices(data=self.data_input, alpha=.05)
        print("Outliers:", outliers)
        print("Indices:", outlier_indices)
        #
        return data_output
    #
    def find_outliers(self, limits_bg1, limits_sig, limits_bg2, threshold=1000):
        data_bg1 = self.data_input[limits_bg1[0]:limits_bg1[1]]
        data_sig = self.data_input[limits_sig[0]:limits_sig[1]]
        data_bg2 = self.data_input[limits_bg2[0]:limits_bg2[1]]

        filtered_bg1 = data_bg1[data_bg1 >= threshold]
        print("Filtered background:", filtered_bg1)




# -*- coding: utf-8 -*-
# Author: Yiping Liu
# Description:
# Version: 1.0
# Last Modified: May 7, 2023


from sympy import *
from public.icemelting import (clegg_and_brimblecombe, spencer, monnin)
from uncertainty.models import FluidPitzer

from database.solid_data import solids
import pprint
import uncertainty.methods as pm

pp = pprint.PrettyPrinter(width=41, compact=True)

x, z = symbols('x,z')


def newton_raphson(x_n, y_n, inputed_f, r_total, molalities, max_iterators):
    inputed_partial_x = diff(inputed_f, x)
    inputed_partial_y = diff(inputed_f, y)
    tolerance = 10 ** (-6)
    iterate_number = 0

    while iterate_number < max_iterators:
        print("x_{n}: {x_n}; y_{n}: {y_n}".format(n=iterate_number, x_n=float(x_n), y_n=float(y_n)))
        iterate_number += 1
        partial_x = inputed_partial_x.subs({x: x_n, y: y_n}).evalf()
        partial_y = inputed_partial_y.subs({x: x_n, y: y_n}).evalf()
        multiple_num = 1 / (partial_x + partial_y * r_total)
        f = inputed_f.subs({x: x_n, y: y_n}).evalf()
        x_n_plus = x_n - multiple_num * (f + partial_y * (x_n * r_total - y_n))
        y_n_plus = y_n - multiple_num * (r_total * f - partial_x * (x_n * r_total - y_n))

        if abs((x_n_plus - x_n) / x_n) < tolerance and abs((y_n_plus - y_n) / y_n) < tolerance:
            for key, value in molalities.items():
                molalities[key] = value.subs({x: x_n, y: y_n})
            return molalities
        x_n = x_n_plus
        y_n = y_n_plus


def newton_raphson_general(x_n, y_n, f_func, g_func, molalities, max_iterators):
    partial_fx = diff(f_func, x)
    partial_fy = diff(f_func, y)
    partial_gx = diff(g_func, x)
    partial_gy = diff(g_func, y)
    tolerance = 10 ** (-6)
    iterate_number = 0
    while iterate_number < max_iterators:
        print("x_{n}: {x_n}; y_{n}: {y_n}".format(n=iterate_number, x_n=x_n, y_n=y_n))
        iterate_number += 1
        partial_fx = partial_fx.subs({x: x_n, y: y_n}).evalf()
        partial_fy = partial_fy.subs({x: x_n, y: y_n}).evalf()
        partial_gx = partial_gx.subs({x: x_n, y: y_n}).evalf()
        partial_gy = partial_gy.subs({x: x_n, y: y_n}).evalf()
        det_value = partial_fx * partial_gy - partial_fy * partial_gx
        f_value = f_func.subs({x: x_n, y: y_n}).evalf()
        g_value = g_func.subs({x: x_n, y: y_n}).evalf()
        x_n_plus = x_n - (1 / det_value) * (partial_gy * f_value - partial_fy * g_value)
        y_n_plus = y_n - (1 / det_value) * (-partial_gx * f_value + partial_fx * g_value)
        if abs((x_n_plus - x_n) / x_n) < tolerance and abs((y_n_plus - y_n) / y_n) < tolerance:
            for key, value in molalities.items():
                try:
                    molalities[key] = value.subs({x: x_n, y: y_n})
                except AttributeError:
                    molalities[key] = value
            return molalities, y_n
        x_n = x_n_plus
        y_n = y_n_plus


def newton_raphson_three(funcs, x_n, y_n, z_n, molalities, max_iterators):
    f_func = funcs[0]
    g_func = funcs[1]
    h_func = funcs[2]
    a1 = diff(f_func, x)  # partial_fx
    a2 = diff(f_func, y)  # partial_fy
    a3 = diff(f_func, z)  # partial_fz

    b1 = diff(g_func, x)  # partial_gx
    b2 = diff(g_func, y)  # partial_gy
    b3 = diff(g_func, z)  # partial_gz

    c1 = diff(h_func, x)  # partial_hx
    c2 = diff(h_func, y)  # partial_hy
    c3 = diff(h_func, z)  # partial_hz

    tolerance = 10 ** (-6)
    iterate_number = 0
    while iterate_number < max_iterators:
        print("x_{n}: {x_n}; y_{n}: {y_n}; z_{n}: {z_n}".format(n=iterate_number, x_n=x_n, y_n=y_n, z_n=z_n))
        iterate_number += 1
        a1 = a1.subs({x: x_n, y: y_n, z: z_n}).evalf()
        a2 = a2.subs({x: x_n, y: y_n, z: z_n}).evalf()
        a3 = a3.subs({x: x_n, y: y_n, z: z_n}).evalf()
        b1 = b1.subs({x: x_n, y: y_n, z: z_n}).evalf()
        b2 = b2.subs({x: x_n, y: y_n, z: z_n}).evalf()
        b3 = b3.subs({x: x_n, y: y_n, z: z_n}).evalf()
        c1 = c1.subs({x: x_n, y: y_n, z: z_n}).evalf()
        c2 = c2.subs({x: x_n, y: y_n, z: z_n}).evalf()
        c3 = c3.subs({x: x_n, y: y_n, z: z_n}).evalf()

        det_value = a1 * b2 * c3 - a1 * b3 * c2 - a2 * b1 * c3 + a2 * b3 * c1 + a3 * b1 * c2 - a3 * b2 * c1

        f_value = f_func.subs({x: x_n, y: y_n, z: z_n}).evalf()
        g_value = g_func.subs({x: x_n, y: y_n, z: z_n}).evalf()
        h_value = h_func.subs({x: x_n, y: y_n, z: z_n}).evalf()

        x_n_plus = x_n - (1 / det_value) * (
                (b2 * c3 - b3 * c2) * f_value + (-a2 * c3 + a3 * c2) * g_value + (a2 * b3 - a3 * b2) * h_value)
        y_n_plus = y_n - (1 / det_value) * (
                (-b1 * c3 + b3 * c1) * f_value + (a1 * c3 - a3 * c1) * g_value + (-a1 * b3 + a3 * b1) * h_value)
        z_n_plus = z_n - (1 / det_value) * (
                (b1 * c2 - b2 * c1) * f_value + (-a1 * c2 + a2 * c1) * g_value + (a1 * b2 - a2 * b1) * h_value)
        if abs((x_n_plus - x_n) / x_n) < tolerance and abs((y_n_plus - y_n) / y_n) < tolerance and abs(
                (z_n_plus - z_n) / z_n) < tolerance:
            for key, value in molalities.items():
                molalities[key] = value.subs({x: x_n, y: y_n, z: z_n})
            return molalities, z_n
        x_n = x_n_plus
        y_n = y_n_plus
        z_n = z_n_plus


def function_generator(solid, fluid):
    lna_pitzer = fluid.get_water_activity()
    # lnk_potential = 0
    if solid == 'H2O(S)':
        # lnk_ice = clegg_and_brimblecombe(fluid.t)
        # lnk_ice = monnin(fluid.t)
        lnk_ice = spencer(fluid.t)
        f = lnk_ice - lna_pitzer
    else:
        molalities = fluid.get_molalities()

        lnk_potential = pm.get_chemical_potential(solid=solid, temperature=fluid.t)

        print('deltaG:', lnk_potential)

        solid_data = solids[solid]
        print(solid_data)
        lnk_activity = 0
        for species in solid_data.keys():
            # get the stochiometric number of this species first
            sto = solid_data[species]['value']
            if solid_data[species]['type'] == 'cation':
                m_c = molalities[species]
                ln_gamma_c = fluid.get_cation_activity_coefficients(target_cation=species)
                lnk_activity += sto * (
                        ln(m_c) + ln_gamma_c
                )
            elif solid_data[species]['type'] == 'anion':
                m_a = molalities[species]
                ln_gamma_a = fluid.get_anion_activity_coefficients(target_anion=species)
                lnk_activity += sto * (
                        ln(m_a) + ln_gamma_a
                )
            else:
                lnk_activity += sto * lna_pitzer
        f = lnk_potential - lnk_activity
    return f


def invariant_point_input(solid1, solid2, species, t, x_guess, y_guess, max_iterators=100, sulfate=False):
    """
    Calculate the eutectic point between two solid phases.
    :param solid1: [string], the first solid phase.
    :param solid2: [string], the second solid phase.
    :param species: species and corresponding ratios with repect to sodium.
    :param temperature_guess: guess the eutectic temperature, for calculating convenience.
    :param concentration_guess: guess the eutectic concentration of the salt, for calculating convenience.
    :return: eutectic temperature and composition.
    """
    fluid = FluidPitzer(
        t=t,
        dictionary=species,
        database='spencer',
    )

    func_1 = function_generator(solid1, fluid)
    func_2 = function_generator(solid2, fluid)
    return newton_raphson_general(
        x_n=x_guess,
        y_n=y_guess,
        f_func=func_1,
        g_func=func_2,
        molalities=fluid.get_molalities(),
        max_iterators=max_iterators
    )


def invariant_point_input_ternary(solids, molalities, x_n, y_n, z_n):
    """
    Calculate the eutectic point between two solid phases.
    :param solids: a list of solids.
    :param molalities: species and corresponding ratios with repect to sodium.
    :param temperature_guess: guess the eutectic temperature, for calculating convenience.
    :param concentration_guess: guess the eutectic concentration of the salt, for calculating convenience.
    :return: eutectic temperature and composition.
    """
    fluid = FluidPitzer(
        t=z,
        dictionary=molalities,
        database='spencer',

    )
    func_1 = function_generator(solids[0], fluid)
    func_2 = function_generator(solids[1], fluid)
    func_3 = function_generator(solids[2], fluid)
    result = newton_raphson_three(
        funcs=[func_1, func_2, func_3],
        x_n=x_n,
        y_n=y_n,
        z_n=z_n,
        molalities=fluid.get_molalities(),
        max_iterators=100
    )
    return result


def concentration_input(t, species, solid, database='spencer', x_guess=0.01, y_guess=0.01, max_iterators=50,
                        sulfate=False):
    """
    :param t: melting temperature of ice or salts
    :param species: a dictionary, should be in form of {"a species": numeric_ratio,...}.
    :param solid: if the temperature provided is melting temperature of hydrates, then the formula of the hydrate should
    be provided here.
    :param x_guess: guess value of x_n
    :param y_guess: guess value of y_n
    :param database: name of database we will use
    :return:
    """
    print(x_guess, y_guess)
    t_melt = t
    dictionary = species
    fluid = FluidPitzer(
        dictionary=dictionary,
        t=t_melt,
        database=database,
    )
    # print(fluid.get_molalities())

    f_func = function_generator(solid=solid, fluid=fluid)
    g_func = fluid.charge_balance()

    result = newton_raphson_general(
        x_n=x_guess,
        y_n=y_guess,
        f_func=f_func,
        g_func=g_func,
        molalities=fluid.get_molalities(),
        max_iterators=max_iterators
    )
    return result


n = 1
y = symbols('y:' + str(n))
print(y)
v_dict = {
    x: 1.85,  # concentration of Na calculated with the model
    y[0]: 2,
    # y[1]: 0.050009709,
    # y[2]: 0.018006502,
    # y[3]: 0.001333698,
    # y[4]: 0.001159332,
    # y[5]: 0.000121677,
    # y[6]: 0.000139219,
    # y[7]: 0.000423009,
    # y[8]: 0.000329692,
    # y[9]: 2.33738E-05,
    z: 25,
}

u_dict = {
    y[0]: 0.28,
    # y[1]: 0.008029289,
    # y[2]: 0.004282004,
    # y[3]: 0.000493429,
    # y[4]: 0.000567157,
    # y[5]: 1.28302E-05,
    # y[6]: 4.18045E-05,
    # y[7]: 2.47324E-05,
    # y[8]: 0.000122204,
    # y[9]: 2.68774E-06,
    z: 0.2
}

print(u_dict)

fluid = FluidPitzer(
    t=z,
    dictionary={
        'Na+': x,
        'K+': x * y[0],
        # 'Li+': x * y[2],
        # 'Mg+2': x * y[3],
        # 'Sr+2': x * y[4],
        # 'Cs+': x * y[5],
        # 'Rb+': x * y[6],
        # 'Mn+2': x * y[7],
        # 'Zn+2': x * y[8],
        # 'Pb+2': x * y[9],
    },
    database='spencer',
)

# objective function
fun = function_generator('KCl', fluid)

# partial derivative of fun to x,y0,y1,...,z
d_dict = {}
d_dict['d1'] = diff(fun, x)  # partial_fx
d_dict['d2'] = diff(fun, z)  # partial_fz

# partial_fy_i
for i in range(len(v_dict.keys()) - 2):
    d_dict['d{}'.format(i + 3)] = diff(fun, y[i])

# substitute numerical values into partial derivative expressions
for key, value in d_dict.items():
    d_dict[key] = value.subs(v_dict)

# partial to internal standard ∂x/∂z
p_dict = {}
p_dict['p_xz'] = -d_dict['d2'] / d_dict['d1']

# partial to internal standard ∂x/∂y_i, y_i is element/Na ratio
for i in range(len(y)):
    p_dict['p_xy{}'.format(i)] = -d_dict['d{}'.format(i + 3)] / d_dict['d1']

print(p_dict)

import math

sum_v = 0
# uncertainties from element/Na ratios
for i in range(len(y)):
    partial = p_dict['p_xy{}'.format(i)]
    uncertainty = u_dict[y[i]]
    sum_v += (partial * uncertainty) ** 2

# uncertainties from solid melting temperautre
partial_z = p_dict['p_xz']
uncertainty_z = u_dict[z]
sum_v += (partial_z * uncertainty_z) ** 2

# get square root of sum_v
d_x = math.sqrt(sum_v)

print(d_x)
